<?php
/**
 * Instance counter class.
 *
 * @since      1.0.0
 *
 * @package    Podcast_Player
 */

namespace Podcast_Player\Frontend\Inc;

/**
 * Load required font icons.
 *
 * @since 6.0.0
 */
class Icon_Loader {

	/**
	 * Holds the instance of this class.
	 *
	 * @since  6.0.0
	 * @access protected
	 * @var    object
	 */
	protected static $instance = null;

	/**
	 * Holds all required font icons.
	 *
	 * @since  6.0.0
	 * @access private
	 * @var    array
	 */
	private $icons = array();

	/**
	 * Constructor method.
	 *
	 * @since  6.0.0
	 */
	public function __construct() {}

	/**
	 * Adds a font icon to icons array.
	 *
	 * @since  6.0.0
	 *
	 * @param str $icon Icon to be added.
	 */
	public function add( $icon ) {
		if ( ! in_array( $icon, $this->icons, true ) ) {
			$this->icons[] = $icon;
		}
	}

	/**
	 * Adds a font icon to footer the web page.
	 *
	 * @since  6.0.0
	 */
	public function add_icons() {
		if ( empty( $this->icons ) ) {
			return;
		}

		$icons = '<svg style="position: absolute; width: 0; height: 0; overflow: hidden;" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><defs>';

		$icons_def = $this->get_font_icons_def();
		foreach ( $this->icons as $icon ) {
			if ( isset( $icons_def[ $icon ] ) ) {
				$icons .= $icons_def[ $icon ];
			}
		}

		$icons .= '</defs></svg>';
		echo $icons; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
	}

	/**
	 * SVG icons definition.
	 *
	 * @since 6.0.0
	 */
	public function get_font_icons_def() {
		/*
		* Icons Licensing Information.
		* 1. Feather free svg icons. [https://github.com/feathericons] Cole Bemis - MIT
		* 2. Icomoon free svg icons. [https://icomoon.io] Keyamoon - GPL / CC BY 4.0
		* 3. Entypo+ SVG Icons [http://www.entypo.com/] Daniel Bruce — CC BY-SA 4.0
		* 4. Font Awesomw svg icons. [https://github.com/FortAwesome/Font-Awesome] Dave Gandy - CC BY 4.0
		* 5. Material design svg icons. [https://github.com/google/material-design-icons] Google - Apache-2.0
		* 6. Carbon svg icons. [https://github.com/carbon-design-system/carbon/tree/master/packages/icons] IBM - Apache-2.0
		* 7. Fluent UI System Icons. [https://iconify.design/icon-sets/fluent/] Microsoft Corporation - MIT
		*/
		return apply_filters(
			'pp_icon_fonts_def',
			array(
				'pp-menu'          => '<symbol id="icon-pp-menu" viewBox="0 0 32 32"><path d="M4 17.333h24c0.736 0 1.333-0.597 1.333-1.333s-0.597-1.333-1.333-1.333h-24c-0.736 0-1.333 0.597-1.333 1.333s0.597 1.333 1.333 1.333zM4 9.333h24c0.736 0 1.333-0.597 1.333-1.333s-0.597-1.333-1.333-1.333h-24c-0.736 0-1.333 0.597-1.333 1.333s0.597 1.333 1.333 1.333zM4 25.333h24c0.736 0 1.333-0.597 1.333-1.333s-0.597-1.333-1.333-1.333h-24c-0.736 0-1.333 0.597-1.333 1.333s0.597 1.333 1.333 1.333z"></path></symbol>',
				'pp-x'             => '<symbol id="icon-pp-x" viewBox="0 0 32 32"><path d="M31.708 25.708c-0-0-0-0-0-0l-9.708-9.708 9.708-9.708c0-0 0-0 0-0 0.105-0.105 0.18-0.227 0.229-0.357 0.133-0.356 0.057-0.771-0.229-1.057l-4.586-4.586c-0.286-0.286-0.702-0.361-1.057-0.229-0.13 0.048-0.252 0.124-0.357 0.228 0 0-0 0-0 0l-9.708 9.708-9.708-9.708c-0-0-0-0-0-0-0.105-0.104-0.227-0.18-0.357-0.228-0.356-0.133-0.771-0.057-1.057 0.229l-4.586 4.586c-0.286 0.286-0.361 0.702-0.229 1.057 0.049 0.13 0.124 0.252 0.229 0.357 0 0 0 0 0 0l9.708 9.708-9.708 9.708c-0 0-0 0-0 0-0.104 0.105-0.18 0.227-0.229 0.357-0.133 0.355-0.057 0.771 0.229 1.057l4.586 4.586c0.286 0.286 0.702 0.361 1.057 0.229 0.13-0.049 0.252-0.124 0.357-0.229 0-0 0-0 0-0l9.708-9.708 9.708 9.708c0 0 0 0 0 0 0.105 0.105 0.227 0.18 0.357 0.229 0.356 0.133 0.771 0.057 1.057-0.229l4.586-4.586c0.286-0.286 0.362-0.702 0.229-1.057-0.049-0.13-0.124-0.252-0.229-0.357z"></path></symbol>',
				'pp-facebook'      => '<symbol id="icon-pp-facebook" viewBox="0 0 32 32"><path d="M29 0h-26c-1.65 0-3 1.35-3 3v26c0 1.65 1.35 3 3 3h13v-14h-4v-4h4v-2c0-3.306 2.694-6 6-6h4v4h-4c-1.1 0-2 0.9-2 2v2h6l-1 4h-5v14h9c1.65 0 3-1.35 3-3v-26c0-1.65-1.35-3-3-3z"></path></symbol>',
				'pp-linkedin'      => '<symbol id="icon-pp-linkedin" viewBox="0 0 32 32"><path d="M29 0h-26c-1.65 0-3 1.35-3 3v26c0 1.65 1.35 3 3 3h26c1.65 0 3-1.35 3-3v-26c0-1.65-1.35-3-3-3zM12 26h-4v-14h4v14zM10 10c-1.106 0-2-0.894-2-2s0.894-2 2-2c1.106 0 2 0.894 2 2s-0.894 2-2 2zM26 26h-4v-8c0-1.106-0.894-2-2-2s-2 0.894-2 2v8h-4v-14h4v2.481c0.825-1.131 2.087-2.481 3.5-2.481 2.488 0 4.5 2.238 4.5 5v9z"></path></symbol>',
				'pp-mail'          => '<symbol id="icon-pp-mail" viewBox="0 0 32 32"><path d="M26.667 0h-21.333c-2.934 0-5.334 2.4-5.334 5.334v21.332c0 2.936 2.4 5.334 5.334 5.334h21.333c2.934 0 5.333-2.398 5.333-5.334v-21.332c0-2.934-2.399-5.334-5.333-5.334zM26.667 4c0.25 0 0.486 0.073 0.688 0.198l-11.355 9.388-11.355-9.387c0.202-0.125 0.439-0.198 0.689-0.198h21.333zM5.334 28c-0.060 0-0.119-0.005-0.178-0.013l7.051-9.78-0.914-0.914-7.293 7.293v-19.098l12 14.512 12-14.512v19.098l-7.293-7.293-0.914 0.914 7.051 9.78c-0.058 0.008-0.117 0.013-0.177 0.013h-21.333z"></path></symbol>',
				'pp-twitter'       => '<symbol id="icon-pp-twitter" viewBox="0 0 32 32"><path d="M32 7.075c-1.175 0.525-2.444 0.875-3.769 1.031 1.356-0.813 2.394-2.1 2.887-3.631-1.269 0.75-2.675 1.3-4.169 1.594-1.2-1.275-2.906-2.069-4.794-2.069-3.625 0-6.563 2.938-6.563 6.563 0 0.512 0.056 1.012 0.169 1.494-5.456-0.275-10.294-2.888-13.531-6.862-0.563 0.969-0.887 2.1-0.887 3.3 0 2.275 1.156 4.287 2.919 5.463-1.075-0.031-2.087-0.331-2.975-0.819 0 0.025 0 0.056 0 0.081 0 3.181 2.263 5.838 5.269 6.437-0.55 0.15-1.131 0.231-1.731 0.231-0.425 0-0.831-0.044-1.237-0.119 0.838 2.606 3.263 4.506 6.131 4.563-2.25 1.762-5.075 2.813-8.156 2.813-0.531 0-1.050-0.031-1.569-0.094 2.913 1.869 6.362 2.95 10.069 2.95 12.075 0 18.681-10.006 18.681-18.681 0-0.287-0.006-0.569-0.019-0.85 1.281-0.919 2.394-2.075 3.275-3.394z"></path></symbol>',
				'pp-download'      => '<symbol id="icon-pp-download" viewBox="0 0 32 32"><path d="M16 18l8-8h-6v-8h-4v8h-6zM23.273 14.727l-2.242 2.242 8.128 3.031-13.158 4.907-13.158-4.907 8.127-3.031-2.242-2.242-8.727 3.273v8l16 6 16-6v-8z"></path></symbol>',
				'pp-refresh'       => '<symbol id="icon-pp-refresh" viewBox="0 0 32 32"><path d="M32 16c-0.040-2.089-0.493-4.172-1.331-6.077-0.834-1.906-2.046-3.633-3.533-5.060-1.486-1.428-3.248-2.557-5.156-3.302-1.906-0.748-3.956-1.105-5.981-1.061-2.025 0.040-4.042 0.48-5.885 1.292-1.845 0.809-3.517 1.983-4.898 3.424s-2.474 3.147-3.193 4.994c-0.722 1.846-1.067 3.829-1.023 5.79 0.040 1.961 0.468 3.911 1.254 5.694 0.784 1.784 1.921 3.401 3.316 4.736 1.394 1.336 3.046 2.391 4.832 3.085 1.785 0.697 3.701 1.028 5.598 0.985 1.897-0.040 3.78-0.455 5.502-1.216 1.723-0.759 3.285-1.859 4.574-3.208 1.29-1.348 2.308-2.945 2.977-4.67 0.407-1.046 0.684-2.137 0.829-3.244 0.039 0.002 0.078 0.004 0.118 0.004 1.105 0 2-0.895 2-2 0-0.056-0.003-0.112-0.007-0.167h0.007zM28.822 21.311c-0.733 1.663-1.796 3.169-3.099 4.412s-2.844 2.225-4.508 2.868c-1.663 0.646-3.447 0.952-5.215 0.909-1.769-0.041-3.519-0.429-5.119-1.14-1.602-0.708-3.053-1.734-4.25-2.991s-2.141-2.743-2.76-4.346c-0.621-1.603-0.913-3.319-0.871-5.024 0.041-1.705 0.417-3.388 1.102-4.928 0.683-1.541 1.672-2.937 2.883-4.088s2.642-2.058 4.184-2.652c1.542-0.596 3.192-0.875 4.832-0.833 1.641 0.041 3.257 0.404 4.736 1.064 1.48 0.658 2.82 1.609 3.926 2.774s1.975 2.54 2.543 4.021c0.57 1.481 0.837 3.064 0.794 4.641h0.007c-0.005 0.055-0.007 0.11-0.007 0.167 0 1.032 0.781 1.88 1.784 1.988-0.195 1.088-0.517 2.151-0.962 3.156z"></path></symbol>',
				'pp-show-list'     => '<symbol id="icon-pp-show-list" viewBox="0 0 44 32"><path d="M0 6h28v6h-28v-6zM0 14h28v6h-28v-6zM0 22h28v6h-28v-6z"></path><path d="M31 14l6 6 6-6z"></path></symbol>',
				'pp-hide-list'     => '<symbol id="icon-pp-hide-list" viewBox="0 0 44 32"><path d="M0 6h28v6h-28v-6zM0 14h28v6h-28v-6zM0 22h28v6h-28v-6z"></path><path d="M31 20l6-6 6 6z"></path></symbol>',
				'pp-maximize'      => '<symbol id="icon-pp-maximize" viewBox="0 0 32 32"><path d="M32 0h-13l5 5-6 6 3 3 6-6 5 5z"></path><path d="M32 32v-13l-5 5-6-6-3 3 6 6-5 5z"></path><path d="M0 32h13l-5-5 6-6-3-3-6 6-5-5z"></path><path d="M0 0v13l5-5 6 6 3-3-6-6 5-5z"></path></symbol>',
				'pp-minimize'      => '<symbol id="icon-pp-minimize" viewBox="0 0 32 32"><path d="M18 14h13l-5-5 6-6-3-3-6 6-5-5z"></path><path d="M18 18v13l5-5 6 6 3-3-6-6 5-5z"></path><path d="M14 18h-13l5 5-6 6 3 3 6-6 5 5z"></path><path d="M14 14v-13l-5 5-6-6-3 3 6 6-5 5z"></path></symbol>',
				'pp-podcast'       => '<symbol id="icon-pp-podcast" viewBox="0 0 32 32"><path d="M14.4 18.768c-0.963-0.564-1.6-1.593-1.6-2.771 0-1.767 1.433-3.2 3.2-3.2s3.2 1.433 3.2 3.2c0 1.178-0.637 2.208-1.585 2.763l-0.015 0.008v13.232h-3.2v-13.232zM22.784 22.784l-2.256-2.256c1.173-1.158 1.9-2.766 1.9-4.544 0-3.526-2.858-6.384-6.384-6.384-0.015 0-0.031 0-0.046 0h0.002c-0.002 0-0.003 0-0.005 0-3.535 0-6.4 2.865-6.4 6.4 0 1.769 0.717 3.37 1.877 4.528l0 0-2.256 2.256c-1.735-1.737-2.808-4.135-2.808-6.784 0-5.302 4.298-9.6 9.6-9.6s9.6 4.298 9.6 9.6c0 2.649-1.073 5.048-2.808 6.785l0-0zM27.312 27.312l-2.256-2.256c2.318-2.317 3.752-5.518 3.752-9.054 0-7.069-5.731-12.8-12.8-12.8s-12.8 5.731-12.8 12.8c0 3.536 1.434 6.737 3.752 9.054l0 0-2.272 2.256c-2.896-2.896-4.688-6.896-4.688-11.315 0-8.837 7.163-16 16-16s16 7.163 16 16c0 4.419-1.792 8.42-4.688 11.315l-0 0z"></path></symbol>',
				'pp-headphones'    => '<symbol id="icon-pp-headphones" viewBox="0 0 32 32"><path d="M25.6 12.8c0-5.302-4.298-9.6-9.6-9.6s-9.6 4.298-9.6 9.6v0 17.6h-3.2c-1.767 0-3.2-1.433-3.2-3.2v0-6.4c0-1.767 1.433-3.2 3.2-3.2v0-4.8c0-7.069 5.731-12.8 12.8-12.8s12.8 5.731 12.8 12.8v0 4.8c1.767 0 3.2 1.433 3.2 3.2v0 6.4c0 1.767-1.433 3.2-3.2 3.2v0h-3.2v-17.6zM19.2 16h4.8v16h-4.8v-16zM8 16h4.8v16h-4.8v-16z"></path></symbol>',
				'pp-tab'           => '<symbol id="icon-pp-tab" viewBox="0 0 32 32"><path d="M6 2v24h24v-24h-24zM28 24h-20v-20h20v20zM4 28v-21l-2-2v25h25l-2-2h-21z"></path><path d="M11 8l5 5-6 6 3 3 6-6 5 5v-13z"></path></symbol>',
				'pp-text'          => '<symbol id="icon-pp-text" viewBox="0 0 32 32"><path d="M24.8 17.6h-17.6c-0.44 0-0.8 0.36-0.8 0.8v1.6c0 0.442 0.36 0.8 0.8 0.8h17.6c0.442 0 0.8-0.358 0.8-0.8v-1.6c0-0.44-0.358-0.8-0.8-0.8zM24.8 11.2h-17.6c-0.44 0-0.8 0.36-0.8 0.8v1.6c0 0.442 0.36 0.8 0.8 0.8h17.6c0.442 0 0.8-0.358 0.8-0.8v-1.6c0-0.44-0.358-0.8-0.8-0.8zM16.8 24h-9.6c-0.44 0-0.8 0.36-0.8 0.8v1.6c0 0.442 0.36 0.8 0.8 0.8h9.6c0.442 0 0.8-0.358 0.8-0.8v-1.6c0-0.44-0.358-0.8-0.8-0.8zM24.8 4.8h-17.6c-0.44 0-0.8 0.36-0.8 0.8v1.6c0 0.442 0.36 0.8 0.8 0.8h17.6c0.442 0 0.8-0.358 0.8-0.8v-1.6c0-0.44-0.358-0.8-0.8-0.8z"></path></symbol>',
				'pp-drop-down'     => '<symbol id="icon-pp-drop-down" viewBox="0 0 32 32"><path d="M7.226 12.077c0.698-0.714 1.669-0.77 2.522 0l6.253 5.995 6.253-5.995c0.853-0.77 1.826-0.714 2.518 0 0.698 0.712 0.653 1.915 0 2.584-0.65 0.669-7.512 7.203-7.512 7.203-0.347 0.357-0.803 0.536-1.259 0.536s-0.912-0.179-1.262-0.536c0 0-6.859-6.534-7.512-7.203-0.654-0.669-0.698-1.872 0-2.584z"></path></symbol>',
				'pp-dots'          => '<symbol id="icon-pp-dots" viewBox="0 0 32 32"><path d="M16.002 12.48c-1.944 0-3.522 1.576-3.522 3.52s1.578 3.52 3.522 3.52c1.944 0 3.518-1.576 3.518-3.52s-1.574-3.52-3.518-3.52zM4.802 12.48c-1.944 0-3.522 1.576-3.522 3.52s1.578 3.52 3.522 3.52c1.944 0 3.518-1.578 3.518-3.52s-1.574-3.52-3.518-3.52zM27.202 12.48c-1.944 0-3.522 1.576-3.522 3.52s1.578 3.52 3.522 3.52c1.944 0 3.518-1.576 3.518-3.52s-1.574-3.52-3.518-3.52z"></path></symbol>',
				'pp-circle'        => '<symbol id="icon-pp-circle" viewBox="0 0 32 32"><path d="M16 0.64c-8.483 0-15.36 6.877-15.36 15.36s6.877 15.36 15.36 15.36c8.482 0 15.36-6.877 15.36-15.362 0-8.482-6.878-15.358-15.36-15.358zM16 28.158c-6.715 0-12.16-5.443-12.16-12.16s5.443-12.158 12.16-12.158c6.715 0 12.162 5.443 12.162 12.16s-5.446 12.158-12.162 12.158z"></path></symbol>',
				'pp-copy'          => '<symbol id="icon-pp-copy" viewBox="0 0 28 32"><path d="M27.121 4.121l-3.243-3.243c-0.543-0.543-1.293-0.879-2.121-0.879h-10.757c-1.657 0-3 1.343-3 3v3h-5c-1.657 0-3 1.343-3 3v20c0 1.657 1.343 3 3 3h14c1.657 0 3-1.343 3-3v-3h5c1.657 0 3-1.343 3-3v-16.757c0-0.828-0.336-1.578-0.879-2.121v0zM16.625 29h-13.25c-0.207 0-0.375-0.168-0.375-0.375v0-19.25c0-0.207 0.168-0.375 0.375-0.375v0h4.625v14c0 1.657 1.343 3 3 3h6v2.625c0 0.207-0.168 0.375-0.375 0.375v0zM24.625 23h-13.25c-0.207 0-0.375-0.168-0.375-0.375v0-19.25c0-0.207 0.168-0.375 0.375-0.375v0h6.625v5.5c0 0.828 0.672 1.5 1.5 1.5h5.5v12.625c0 0.207-0.168 0.375-0.375 0.375v0zM25 7h-4v-4h0.602c0.099 0 0.195 0.039 0.265 0.11l3.023 3.023c0.068 0.068 0.11 0.162 0.11 0.265 0 0 0 0 0 0v0 0.602z"></path></symbol>',
				'pp-arrow-up'      => '<symbol id="icon-pp-arrow-up" viewBox="0 0 32 32"><path d="M30.054 23.768l-2.964 2.946c-0.446 0.446-1.161 0.446-1.607 0l-9.482-9.482-9.482 9.482c-0.446 0.446-1.161 0.446-1.607 0l-2.964-2.946c-0.446-0.446-0.446-1.179 0-1.625l13.25-13.232c0.446-0.446 1.161-0.446 1.607 0l13.25 13.232c0.446 0.446 0.446 1.179 0 1.625z"></path></symbol>',
				'pp-search'        => '<symbol id="icon-pp-search" viewBox="0 0 30 32"><path d="M20.571 14.857c0-4.411-3.589-8-8-8s-8 3.589-8 8 3.589 8 8 8 8-3.589 8-8zM29.714 29.714c0 1.25-1.036 2.286-2.286 2.286-0.607 0-1.196-0.25-1.607-0.679l-6.125-6.107c-2.089 1.446-4.589 2.214-7.125 2.214-6.946 0-12.571-5.625-12.571-12.571s5.625-12.571 12.571-12.571 12.571 5.625 12.571 12.571c0 2.536-0.768 5.036-2.214 7.125l6.125 6.125c0.411 0.411 0.661 1 0.661 1.607z"></path></symbol>',
				'pp-tune'          => '<symbol id="icon-pp-tune" viewBox="0 0 32 32"><path d="M20 12v-8h2.688v2.688h5.313v2.625h-5.313v2.688h-2.688zM28 17.313h-13.313v-2.625h13.313v2.625zM9.313 12h2.688v8h-2.688v-2.688h-5.313v-2.625h5.313v-2.688zM17.313 28h-2.625v-8h2.625v2.688h10.688v2.625h-10.688v2.688zM4 6.688h13.313v2.625h-13.313v-2.625zM4 22.688h8v2.625h-8v-2.625z"></path></symbol>',
				'pp-apple'         => '<symbol id="icon-pp-apple" viewBox="0 0 32 32"><defs><linearGradient id="a"><stop offset="0" stop-color="#822cbe"/><stop offset="1" stop-color="#d772fb"/></linearGradient><linearGradient id="b" x1="48.95" x2="48.68" y1="32.40" xlink:href="#a" gradientUnits="userSpaceOnUse" gradientTransform="translate(-33)"/></defs><path fill="url(#b)" d="M7.119 0h17.762c3.932 0 7.119 3.187 7.119 7.119v17.762c0 3.932-3.187 7.119-7.119 7.119h-17.762c-3.932 0-7.119-3.187-7.119-7.119v-17.762c0-3.932 3.187-7.119 7.119-7.119z"></path><path fill="#fff" style="fill: var(--color2, #fff)" d="M15.023 28.015c-0.955-0.341-1.159-0.805-1.551-3.524-0.455-3.161-0.555-5.117-0.291-5.737 0.35-0.821 1.299-1.287 2.627-1.293 1.317-0.005 2.275 0.465 2.627 1.293 0.265 0.619 0.165 2.576-0.29 5.737-0.309 2.208-0.48 2.766-0.907 3.141-0.587 0.519-1.419 0.663-2.208 0.384zM10.945 24.772c-3.296-1.621-5.408-4.365-6.176-8.015-0.192-0.942-0.224-3.189-0.043-4.053 0.48-2.32 1.397-4.134 2.923-5.739 2.197-2.317 5.024-3.541 8.171-3.541 3.115 0 5.931 1.202 8.075 3.45 1.632 1.696 2.549 3.491 3.019 5.856 0.16 0.787 0.16 2.933 0.011 3.819-0.491 2.799-2.048 5.348-4.32 7.061-0.811 0.612-2.795 1.681-3.115 1.681-0.117 0-0.128-0.122-0.075-0.613 0.096-0.789 0.192-0.954 0.64-1.141 0.715-0.299 1.931-1.165 2.677-1.914 1.291-1.28 2.24-2.955 2.677-4.715 0.277-1.099 0.245-3.541-0.064-4.672-0.971-3.595-3.904-6.389-7.499-7.136-1.045-0.213-2.944-0.213-4 0-3.637 0.747-6.645 3.68-7.563 7.371-0.245 1.003-0.245 3.445 0 4.448 0.608 2.443 2.187 4.683 4.256 6.016 0.405 0.267 0.896 0.544 1.099 0.629 0.448 0.192 0.544 0.352 0.629 1.141 0.053 0.48 0.043 0.619-0.075 0.619-0.075 0-0.619-0.235-1.195-0.512zM10.988 20.433c-1.109-0.885-2.091-2.455-2.496-3.996-0.245-0.93-0.245-2.699 0.011-3.627 0.672-2.505 2.517-4.446 5.077-5.358 0.875-0.309 2.816-0.379 3.893-0.141 3.712 0.827 6.347 4.544 5.835 8.231-0.203 1.486-0.715 2.706-1.621 3.84-0.448 0.573-1.536 1.534-1.728 1.534-0.032 0-0.064-0.363-0.064-0.804v-0.806l0.555-0.661c2.091-2.505 1.941-6.003-0.341-8.299-0.885-0.894-1.909-1.419-3.232-1.661-0.853-0.158-1.035-0.158-1.931-0.011-1.36 0.222-2.414 0.749-3.349 1.675-2.293 2.272-2.443 5.789-0.352 8.295l0.55 0.661v0.811c0 0.448-0.035 0.811-0.079 0.811-0.043 0-0.352-0.213-0.683-0.48zM14.689 16.078c-0.949-0.442-1.461-1.275-1.472-2.361 0-0.976 0.544-1.827 1.483-2.325 0.597-0.314 1.653-0.314 2.251 0.002 0.651 0.338 1.184 0.994 1.387 1.679 0.619 2.103-1.611 3.947-3.627 3.005z"></path><path fill="#fff" style="fill: var(--color2, #fff)" d="M18.649 13.831c0 1.469-1.191 2.66-2.66 2.66s-2.66-1.191-2.66-2.66c0-1.469 1.191-2.66 2.66-2.66s2.66 1.191 2.66 2.66z"></path><path fill="#fff" style="fill: var(--color2, #fff)" d="M16.243 17.522c0.185 0.025 0.551 0.073 0.891 0.181 0.341 0.107 0.651 0.274 0.892 0.446 0.235 0.171 0.405 0.348 0.512 0.563s0.16 0.471 0.192 0.891c0.021 0.42 0.021 1.003-0.075 2.091-0.096 1.085-0.288 2.675-0.437 3.694-0.149 1.024-0.256 1.472-0.395 1.803-0.139 0.341-0.32 0.565-0.523 0.725s-0.437 0.256-0.661 0.309c-0.224 0.053-0.427 0.053-0.619 0.053s-0.373 0-0.619-0.053-0.565-0.149-0.8-0.341c-0.245-0.192-0.405-0.469-0.533-0.832s-0.224-0.8-0.352-1.675c-0.128-0.875-0.299-2.176-0.405-3.168s-0.149-1.675-0.158-2.133c-0.011-0.459 0.011-0.686 0.064-0.907 0.053-0.213 0.139-0.416 0.256-0.587s0.271-0.32 0.427-0.437c0.153-0.117 0.309-0.203 0.497-0.269 0.186-0.075 0.409-0.139 0.683-0.192 0.277-0.053 0.597-0.107 0.762-0.128 0.164-0.025 0.164-0.025 0.348 0z"></path></symbol>',
				'pp-google'        => '<symbol id="icon-pp-google" viewBox="0 0 32 32"><path fill="#0066d9" style="fill: var(--color4, #0066d9)" d="M4 14.909v2.182c0 1.105-0.895 2-2 2s-2-0.895-2-2v-2.182c0-1.105 0.895-2 2-2s2 0.895 2 2z"></path><path fill="#4285f4" style="fill: var(--color5, #4285f4)" d="M28.001 14.966c-0-0.017-0.001-0.037-0.001-0.057v-0c0-1.105 0.895-2 2-2s2 0.895 2 2c0 0.019-0 0.038-0.001 0.057h0.001v2.182h-0.001c-0.030 1.078-0.914 1.943-1.999 1.943s-1.969-0.865-1.999-1.943h-0.001v-2.182h0.001z"></path><path fill="#ea4335" style="fill: var(--color6, #ea4335)" d="M10.909 21.455v2.182c0 1.105-0.895 2-2 2s-2-0.895-2-2v-2.182c0-1.105 0.895-2 2-2s2 0.895 2 2zM10.909 8.364v7.136h-0.001c-0.024 1.084-0.91 1.955-1.999 1.955s-1.975-0.871-1.999-1.955h-0.001v-7.136c0-1.105 0.895-2 2-2s2 0.895 2 2z"></path><path fill="#34a853" style="fill: var(--color7, #34a853)" d="M21.091 10.545c0 1.105 0.895 2 2 2s2-0.895 2-2v-2.182c0-1.105-0.895-2-2-2s-2 0.895-2 2z"></path><path fill="#fab908" style="fill: var(--color8, #fab908)" d="M14 4.182c0 1.104 0.895 2 2 2s2-0.896 2-2v-2.182c0-1.105-0.895-2-2-2s-2 0.895-2 2zM14 27.818c0-1.105 0.895-2 2-2s2 0.895 2 2v2.182c0 1.105-0.895 2-2 2s-2-0.895-2-2z"></path><path fill="#34a853" style="fill: var(--color7, #34a853)" d="M21.091 16.545c0-1.105 0.895-2 2-2s2 0.895 2 2v7.091c0 1.105-0.895 2-2 2s-2-0.895-2-2z"></path><path fill="#fab908" style="fill: var(--color8, #fab908)" d="M18 10.182v11.636c0 1.105-0.895 2-2 2s-2-0.895-2-2v-11.636c0-1.105 0.895-2 2-2s2 0.895 2 2z"></path></symbol>',
				'pp-spotify'       => '<symbol id="icon-pp-spotify" viewBox="0 0 32 32"><path fill="#1ed760" style="fill: var(--color1, #1ed760)" d="M16 0c-8.8 0-16 7.2-16 16s7.2 16 16 16 16-7.2 16-16-7.12-16-16-16zM23.361 23.12c-0.32 0.479-0.88 0.64-1.361 0.32-3.76-2.32-8.48-2.801-14.081-1.521-0.557 0.163-1.039-0.239-1.199-0.719-0.16-0.561 0.24-1.040 0.72-1.2 6.080-1.361 11.36-0.8 15.52 1.76 0.56 0.24 0.639 0.879 0.401 1.36zM25.281 18.72c-0.401 0.56-1.121 0.8-1.683 0.4-4.319-2.64-10.879-3.44-15.919-1.84-0.639 0.16-1.36-0.16-1.52-0.8s0.16-1.361 0.8-1.521c5.84-1.759 13.040-0.877 18 2.161 0.481 0.241 0.72 1.040 0.321 1.6zM25.441 14.24c-5.121-3.040-13.681-3.36-18.561-1.839-0.8 0.239-1.6-0.241-1.84-0.961-0.24-0.801 0.24-1.6 0.96-1.841 5.68-1.68 15.040-1.36 20.961 2.161 0.719 0.4 0.959 1.36 0.559 2.080-0.399 0.561-1.36 0.799-2.079 0.4z"></path></symbol>',
				'pp-rotate-cw'     => '<symbol id="icon-pp-rotate-cw" viewBox="0 0 32 32"><path d="M26.4 4.8c0.442 0 0.8 0.358 0.8 0.8v0 6.4c0 0.442-0.358 0.8-0.8 0.8v0h-6.4c-0.442 0-0.8-0.358-0.8-0.8s0.358-0.8 0.8-0.8v0h5.2c-2.049-2.917-5.399-4.8-9.188-4.8-4.372 0-8.158 2.506-10.001 6.159l-0.029 0.064c-0.134 0.265-0.404 0.444-0.717 0.444-0.442 0-0.8-0.358-0.8-0.8 0-0.13 0.031-0.252 0.086-0.361l-0.002 0.005c2.14-4.249 6.467-7.114 11.464-7.114 3.812 0 7.234 1.667 9.578 4.311l0.012 0.013v-3.523c0-0.442 0.358-0.8 0.8-0.8v0zM21.605 16c-1.496 0-2.555 0.699-3.194 1.808-0.602 1.045-0.806 2.411-0.806 3.792s0.205 2.747 0.806 3.792c0.638 1.109 1.698 1.808 3.194 1.808s2.555-0.699 3.194-1.808c0.602-1.045 0.806-2.411 0.806-3.792s-0.205-2.747-0.806-3.792c-0.638-1.109-1.698-1.808-3.194-1.808zM19.205 21.6c0-1.269 0.195-2.304 0.592-2.995 0.363-0.629 0.904-1.005 1.808-1.005s1.445 0.376 1.808 1.005c0.397 0.691 0.592 1.726 0.592 2.995s-0.195 2.304-0.592 2.995c-0.363 0.629-0.904 1.005-1.808 1.005s-1.445-0.376-1.808-1.005c-0.397-0.691-0.592-1.726-0.592-2.995zM8.234 18.566c0.145 0.145 0.345 0.234 0.566 0.234 0.219 0 0.418-0.088 0.562-0.231l-0 0 0.003-0.002c0.196-0.17 0.415-0.324 0.649-0.454l0.020-0.010c0.568-0.317 1.245-0.504 1.966-0.504h0c0.957 0 1.568 0.259 1.923 0.56 0.297 0.237 0.485 0.598 0.485 1.003 0 0.006-0 0.011-0 0.017v-0.001c-0.016 0.656-0.646 1.621-2.408 1.621-0.442 0-0.8 0.358-0.8 0.8s0.358 0.8 0.8 0.8v0c1.762 0 2.392 0.963 2.408 1.621 0 0.005 0 0.010 0 0.016 0 0.405-0.188 0.767-0.482 1.001l-0.003 0.002c-0.354 0.301-0.966 0.56-1.923 0.56-0.94-0-1.806-0.317-2.498-0.85l0.010 0.007c-0.055-0.043-0.103-0.084-0.149-0.126l0.002 0.002c-0.145-0.144-0.344-0.233-0.565-0.233s-0.422 0.090-0.566 0.235l-0 0c-0.539 0.541 0 1.133 0 1.133l0.090 0.082c0.277 0.24 0.586 0.457 0.916 0.64l0.028 0.014c0.638 0.349 1.568 0.698 2.733 0.698 1.243 0 2.25-0.339 2.958-0.939 0.71-0.603 1.072-1.434 1.050-2.282-0.024-0.942-0.496-1.795-1.299-2.379 0.805-0.584 1.275-1.437 1.299-2.379 0-0.018 0.001-0.038 0.001-0.059 0-0.893-0.407-1.691-1.045-2.219l-0.005-0.004c-0.709-0.598-1.715-0.939-2.958-0.939-1.314 0.001-2.523 0.446-3.487 1.194l0.013-0.010c-0.097 0.076-0.183 0.149-0.265 0.225l0.003-0.003-0.019 0.016-0.006 0.008-0.003 0.003c-0.145 0.145-0.235 0.345-0.235 0.566s0.090 0.422 0.235 0.566l0 0zM9.365 24.632h-0.002z"></path></symbol>',
				'pp-rotate-ccw'    => '<symbol id="icon-pp-rotate-ccw" viewBox="0 0 32 32"><path d="M4.797 5.6c0-0.442 0.358-0.8 0.8-0.8s0.8 0.358 0.8 0.8v0 3.53c2.356-2.661 5.78-4.33 9.594-4.33 4.997 0 9.324 2.864 11.43 7.041l0.034 0.073c0.054 0.105 0.086 0.229 0.086 0.36 0 0.442-0.358 0.8-0.8 0.8-0.314 0-0.586-0.181-0.717-0.445l-0.002-0.005c-1.872-3.718-5.658-6.224-10.030-6.224-3.789 0-7.139 1.883-9.164 4.764l-0.024 0.036h5.194c0.442 0 0.8 0.358 0.8 0.8s-0.358 0.8-0.8 0.8v0h-6.4c-0.442 0-0.8-0.358-0.8-0.8v0-6.4zM12.8 16.8c-0-0.442-0.358-0.8-0.8-0.8-0.165 0-0.319 0.050-0.446 0.136l0.003-0.002-2.4 1.6c-0.205 0.147-0.337 0.384-0.337 0.653 0 0.442 0.358 0.8 0.8 0.8 0.157 0 0.303-0.045 0.427-0.123l-0.003 0.002 1.157-0.771v8.106c0 0.442 0.358 0.8 0.8 0.8s0.8-0.358 0.8-0.8v0-9.6zM20.005 16c-1.496 0-2.555 0.699-3.194 1.808-0.602 1.045-0.806 2.411-0.806 3.792s0.205 2.747 0.806 3.792c0.638 1.109 1.698 1.808 3.194 1.808s2.555-0.699 3.194-1.808c0.602-1.045 0.806-2.411 0.806-3.792s-0.205-2.747-0.806-3.792c-0.638-1.109-1.698-1.808-3.194-1.808zM17.605 21.6c0-1.269 0.195-2.304 0.592-2.995 0.363-0.629 0.904-1.005 1.808-1.005s1.445 0.376 1.808 1.005c0.397 0.691 0.592 1.726 0.592 2.995s-0.195 2.304-0.592 2.995c-0.363 0.629-0.904 1.005-1.808 1.005s-1.445-0.376-1.808-1.005c-0.397-0.691-0.592-1.726-0.592-2.995z"></path></symbol>',
				'pp-play'          => '<symbol id="icon-pp-play" viewBox="0 0 32 32"><path d="M7 28c-0.552 0-1-0.448-1-1v0-22c0-0.552 0.448-1 1-1 0.184 0 0.357 0.050 0.506 0.137l-0.005-0.003 19 11c0.3 0.176 0.498 0.498 0.498 0.865s-0.198 0.689-0.494 0.862l-0.005 0.003-19 11c-0.144 0.085-0.316 0.135-0.501 0.135v0z"></path></symbol>',
				'pp-next'          => '<symbol id="icon-pp-next" viewBox="0 0 32 32"><path d="M28 5.333c0-0.736-0.597-1.333-1.333-1.333s-1.333 0.597-1.333 1.333v0 21.333c0 0.736 0.597 1.333 1.333 1.333s1.333-0.597 1.333-1.333v0-21.333z"></path><path d="M4 6.596c0-1.899 2.149-3.003 3.693-1.896l13.96 10.009c0.592 0.429 0.973 1.118 0.973 1.896 0 0.821-0.424 1.542-1.064 1.958l-0.009 0.006-13.96 8.955c-1.553 0.996-3.593-0.119-3.593-1.964v-18.964z"></path></symbol>',
				'pp-previous'      => '<symbol id="icon-pp-previous" viewBox="0 0 32 32"><path d="M3.667 26.667c0 0.736 0.597 1.333 1.333 1.333s1.333-0.597 1.333-1.333v0-21.333c0-0.736-0.597-1.333-1.333-1.333s-1.333 0.597-1.333 1.333v0 21.333z"></path><path d="M27.667 25.404c0 1.899-2.149 3.003-3.693 1.896l-13.96-10.009c-0.592-0.429-0.973-1.118-0.973-1.896 0-0.821 0.424-1.542 1.064-1.958l0.009-0.005 13.96-8.955c1.553-0.997 3.593 0.119 3.593 1.964v18.964z"></path></symbol>',
				'pp-share'         => '<symbol id="icon-pp-share" viewBox="0 0 32 32"><path d="M8.996 5.333h4.619c0.552 0 1 0.448 1 1 0 0.504-0.373 0.921-0.858 0.99l-0.005 0.001-0.136 0.009h-4.619c-0 0-0 0-0 0-1.585 0-2.882 1.229-2.992 2.785l-0.001 0.010-0.007 0.205v12.667c0 0 0 0 0 0 0 1.585 1.229 2.882 2.785 2.992l0.010 0.001 0.205 0.007h12.667c0 0 0 0 0 0 1.585 0 2.882-1.229 2.992-2.785l0.001-0.010 0.007-0.205v-0.664c-0-0.003-0-0.006-0-0.009 0-0.552 0.448-1 1-1 0.508 0 0.927 0.379 0.991 0.869l0.001 0.005 0.008 0.135v0.664c0 0 0 0 0 0 0 2.668-2.089 4.847-4.72 4.992l-0.013 0.001-0.267 0.007h-12.667c-0 0-0 0-0 0-2.668 0-4.847-2.089-4.992-4.72l-0.001-0.013-0.007-0.267v-12.667c0-0 0-0 0-0 0-2.668 2.089-4.847 4.72-4.992l0.013-0.001 0.267-0.007zM19.333 8.725v-3.725c0-0 0-0 0-0.001 0-0.552 0.448-1 1-1 0.219 0 0.421 0.070 0.586 0.189l-0.003-0.002 0.109 0.092 7.992 7.667c0.373 0.357 0.408 0.933 0.103 1.329l-0.103 0.113-7.992 7.669c-0.179 0.173-0.424 0.279-0.693 0.279-0.502 0-0.918-0.37-0.989-0.852l-0.001-0.005-0.009-0.143v-3.736l-0.428-0.008c-3.544-0.011-6.573 1.444-9.153 4.401-0.661 0.757-1.9 0.176-1.741-0.816 1.103-6.853 4.8-10.727 10.92-11.412l0.403-0.040z"></path></symbol>',
				'pp-pause'         => '<symbol id="icon-pp-pause" viewBox="0 0 32 32"><path d="M7.661 4c-1.289 0-2.333 1.045-2.333 2.333v0 19.333c0 1.288 1.045 2.333 2.333 2.333h4.667c1.289 0 2.333-1.045 2.333-2.333v0-19.333c0-1.289-1.045-2.333-2.333-2.333v0h-4.667z"></path><path d="M19.661 4c-1.289 0-2.333 1.045-2.333 2.333v0 19.333c0 1.288 1.045 2.333 2.333 2.333h4.667c1.289 0 2.333-1.045 2.333-2.333v0-19.333c0-1.289-1.045-2.333-2.333-2.333h-4.667z"></path></symbol>',
				'pp-youtube'       => '<symbol id="icon-pp-youtube" viewBox="0 0 32 32"><path fill="#f00" style="fill: var(--color1, #f00)" d="M31.327 8.273c-0.386-1.353-1.431-2.398-2.756-2.777l-0.028-0.007c-2.493-0.668-12.528-0.668-12.528-0.668s-10.009-0.013-12.528 0.668c-1.353 0.386-2.398 1.431-2.777 2.756l-0.007 0.028c-0.443 2.281-0.696 4.903-0.696 7.585 0 0.054 0 0.109 0 0.163l-0-0.008c-0 0.037-0 0.082-0 0.126 0 2.682 0.253 5.304 0.737 7.845l-0.041-0.26c0.386 1.353 1.431 2.398 2.756 2.777l0.028 0.007c2.491 0.669 12.528 0.669 12.528 0.669s10.008 0 12.528-0.669c1.353-0.386 2.398-1.431 2.777-2.756l0.007-0.028c0.425-2.233 0.668-4.803 0.668-7.429 0-0.099-0-0.198-0.001-0.297l0 0.015c0.001-0.092 0.001-0.201 0.001-0.31 0-2.626-0.243-5.196-0.708-7.687l0.040 0.258zM12.812 20.801v-9.591l8.352 4.803z"></path></symbol>',
				'pp-stitcher'      => '<symbol id="icon-pp-stitcher" viewBox="0 0 52 32"><path fill="#f9c516" style="fill: var(--color2, #f9c516)" d="M21.649 18.089c0-7.651 0.009-13.916 0.020-13.921s2.102-0.005 4.647 0.001l4.626 0.010v27.822h-9.294zM32.358 22.289c0-0.054 0.006-0.075 0.012-0.049s0.007 0.071 0 0.097c-0.007 0.027-0.012 0.005-0.012-0.049zM32.36 21.941c0-0.107 0.005-0.148 0.011-0.090s0.006 0.145-0 0.195c-0.006 0.049-0.011 0.002-0.011-0.105zM32.365 15.443c0-3.421 0.003-4.82 0.007-3.11s0.004 4.509 0 6.219c-0.004 1.71-0.007 0.311-0.007-3.11zM32.364 7.235c0-0.834 0.003-1.175 0.008-0.758s0.004 1.099 0 1.517c-0.004 0.417-0.008 0.076-0.008-0.758zM32.364 2.63c0-1.454 0.003-2.053 0.007-1.331s0.004 1.912 0 2.643c-0.004 0.732-0.007 0.141-0.007-1.313z"></path><path fill="#ea8922" style="fill: var(--color3, #ea8922)" d="M32.369 17.023c0.007-6.462 0.012-12.936 0.011-14.386l-0.001-2.637h9.375v28.772h-9.397zM45.513 4.988c1.27-0.004 3.349-0.004 4.62 0s0.231 0.007-2.31 0.007c-2.541 0-3.58-0.003-2.31-0.007z"></path><path fill="#3791c3" style="fill: var(--color4, #3791c3)" d="M0 18.296v-12.146l9.226 0.014-0.008 9.197c-0.005 5.058-0.008 10.521-0.007 12.139l0.002 2.943h-9.212z"></path><path fill="#c23626" style="fill: var(--color5, #c23626)" d="M43.217 16.014v-11.005h9.212v22.010h-9.212z"></path><path fill="#612167" style="fill: var(--color6, #612167)" d="M10.73 15.332v-13.44l2.256-0.007c2.982-0.009 6.373 0.003 6.787 0.024l0.332 0.017v26.847h-9.375z"></path></symbol>',
				'pp-tunein'        => '<symbol id="icon-pp-tunein" viewBox="0 0 32 32"><path fill="#1c203c" stroke="#1c203c" stroke-width="1.1034" stroke-miterlimit="1.4" stroke-linecap="butt" stroke-linejoin="round" style="fill: var(--color1, #1c203c); stroke: var(--color1, #1c203c)" d="M4.626 0.552h22.749c2.25 0 4.074 1.824 4.074 4.074v22.749c0 2.25-1.824 4.074-4.074 4.074h-22.749c-2.25 0-4.074-1.824-4.074-4.074v-22.749c0-2.25 1.824-4.074 4.074-4.074z"></path><path fill="none" stroke="#00ddcf" stroke-width="1.1429" stroke-miterlimit="1.4" stroke-linecap="butt" stroke-linejoin="round" style="stroke: var(--color2, #00ddcf)" d="M6.039 12.684h9.961v9.905h-9.961v-9.905z"></path><path fill="#00ddcf" stroke="#00ddcf" stroke-width="1.1429" stroke-miterlimit="1.4" stroke-linecap="butt" stroke-linejoin="round" style="fill: var(--color2, #00ddcf); stroke: var(--color2, #00ddcf)" d="M16 9.411h9.961v9.905h-9.961v-9.905z"></path><path fill="#00ddcf" style="fill: var(--color2, #00ddcf)" d="M10.581 16.264h0.839c0.017 0 0.031 0.014 0.031 0.031v3.474c0 0.017-0.014 0.031-0.031 0.031h-0.839c-0.017 0-0.031-0.014-0.031-0.031v-3.474c0-0.017 0.014-0.031 0.031-0.031z"></path><path fill="#00ddcf" style="fill: var(--color2, #00ddcf)" d="M9.621 15.465h2.796c0.004 0 0.007 0.003 0.007 0.007v0.769c0 0.004-0.003 0.007-0.007 0.007h-2.796c-0.004 0-0.007-0.003-0.007-0.007v-0.769c0-0.004 0.003-0.007 0.007-0.007z"></path><path fill="#1c203c" style="fill: var(--color1, #1c203c)" d="M20.609 12.983h0.862c0.014 0 0.025 0.011 0.025 0.025v2.729c0 0.014-0.011 0.025-0.025 0.025h-0.862c-0.014 0-0.025-0.011-0.025-0.025v-2.729c0-0.014 0.011-0.025 0.025-0.025z"></path><path fill="#1c203c" style="fill: var(--color1, #1c203c)" d="M19.949 12.136h2.188c0.004 0 0.007 0.003 0.007 0.007v0.816c0 0.004-0.003 0.007-0.007 0.007h-2.188c-0.004 0-0.007-0.003-0.007-0.007v-0.816c0-0.004 0.003-0.007 0.007-0.007z"></path><path fill="#1c203c" style="fill: var(--color1, #1c203c)" d="M19.852 15.762h2.414c0.004 0 0.007 0.003 0.007 0.007v0.811c0 0.004-0.003 0.007-0.007 0.007h-2.414c-0.004 0-0.007-0.003-0.007-0.007v-0.811c0-0.004 0.003-0.007 0.007-0.007z"></path></symbol>',
				'pp-acast'         => '<symbol id="icon-pp-acast" viewBox="0 0 32 32"><path fill="#000" style="fill: var(--color7, #000)" d="M19.2 31.726l12.709-17.829c-1.097-7.863-7.771-13.897-15.909-13.897-8.869 0-16 7.131-16 16s7.131 16 16 16c1.097 0 2.103-0.091 3.2-0.274z"></path><path fill="#fff" style="fill: var(--color8, #fff)" d="M26.514 11.52l-8.046 11.154c-1.006 1.554-3.474 2.743-5.76 2.743-4.023 0-6.674-1.92-6.674-4.846 0-3.657 4.023-5.577 11.886-5.76h0.366v-3.474c0-0.731 0-2.011-1.554-2.011-1.371 0-1.463 0.914-1.463 1.189s0 0.457 0.091 0.64c0.091 0.183 0.091 0.366 0.091 0.549 0 1.189-1.463 2.011-3.749 2.011-2.469 0-3.931-1.189-3.931-3.291 0-2.469 3.2-4.937 9.143-4.937 6.491 0 9.6 3.109 9.6 6.034zM18.286 19.566v-2.651h-0.366c-2.469 0.183-3.383 1.554-3.383 2.743 0 1.097 0.731 1.737 1.829 1.737s1.92-0.823 1.92-1.829z"></path></symbol>',
				'pp-breaker'       => '<symbol id="icon-pp-breaker" viewBox="0 0 32 32"><path fill="#1269ff" style="fill: var(--color9, #1269ff)" d="M13.197 22.75c1.020 3.122 2.66 5.962 4.78 8.375 3.97-0.515 7.462-2.555 9.865-5.515-1.96 0.517-4.017 0.793-6.14 0.793-1.698 0-3.355-0.18-4.953-0.515-1.668-0.34-3.018-1.552-3.553-3.137z"></path><path fill="#5c9dff" style="fill: var(--color10, #5c9dff)" d="M16 31.25c0.67 0 1.33-0.043 1.977-0.128-2.12-2.413-3.76-5.253-4.78-8.375-0.16-0.477-0.248-0.988-0.248-1.518 0-1.192 0.44-2.28 1.165-3.115-2.775 3.102-4.755 6.928-5.62 11.16 2.213 1.258 4.775 1.975 7.505 1.975z"></path><path fill="#9ec6ff" style="fill: var(--color11, #9ec6ff)" d="M1.173 19.582c1 4.152 3.697 7.643 7.32 9.695 0.867-4.238 2.85-8.070 5.63-11.172 0.87-0.995 2.15-1.622 3.575-1.622 0.317 0 0.625 0.030 0.925 0.090-1.59-0.332-3.238-0.51-4.928-0.51-4.588-0.003-8.875 1.285-12.523 3.52z"></path><path fill="#d1e3ff" style="fill: var(--color12, #d1e3ff)" d="M18.625 16.57c-1.59-0.332-3.238-0.51-4.928-0.51-4.59 0-8.877 1.288-12.525 3.523-0.275-1.148-0.422-2.348-0.422-3.582 0-4.755 2.178-9.002 5.588-11.8 7.478 2.258 13.41 8.078 15.848 15.477 0.023 0.065 0.045 0.13 0.065 0.198-0.020-0.065-0.043-0.133-0.065-0.198-0.547-1.573-1.9-2.78-3.56-3.107z"></path><path fill="#fff" style="fill: var(--color8, #fff)" d="M30.497 13.25c0-6.75-5.622-12.5-14.5-12.5-3.665 0-7.030 1.295-9.66 3.45 7.545 2.28 13.517 8.175 15.91 15.672 0.14 0.438 0.267 0.905 0.383 1.355 4.335-0.095 7.867-3.62 7.867-7.977z"></path><path fill="#003dad" style="fill: var(--color13, #003dad)" d="M14.688 18.6c-0.645 0.733-1.002 1.59-0.988 2.633-0.002 0.468 0.063 0.858 0.207 1.277 0.45 1.335 1.592 2.357 2.997 2.64 1.56 0.33 3.14 0.5 4.797 0.5 2.073 0.003 4.035-0.26 5.95-0.767l0.617-0.163 0.32 1.157-0.165 0.203c-2.523 3.102-6.185 5.245-10.352 5.785-0.67 0.085-1.378 0.133-2.072 0.133-2.832-0.008-5.58-0.775-7.877-2.070-3.797-2.15-6.628-5.813-7.678-10.17-0.287-1.19-0.443-2.477-0.445-3.758 0-4.99 2.285-9.447 5.862-12.38 2.73-2.225 6.335-3.607 10.137-3.62 9.217 0 15.25 6.018 15.25 13.25 0 4.775-3.872 8.627-8.607 8.727l-0.595 0.012-0.148-0.578c-0.113-0.438-0.29-1.075-0.425-1.5-0.465-1.323-1.605-2.332-2.995-2.607-0.265-0.052-0.5-0.075-0.78-0.075-1.2 0-2.277 0.53-3.012 1.37zM7.933 3.942c7.115 2.6 12.645 8.233 14.967 15.508 0.090 0.28 0.195 0.648 0.293 0.995 3.668-0.37 6.558-3.447 6.558-7.195 0-6.27-5.22-11.75-13.75-11.75-3.027-0.010-5.755 0.873-8.068 2.442zM1.698 18.41c3.54-1.955 7.68-3.092 12-3.1 1.728 0 3.45 0.188 5.075 0.525 0.4 0.080 0.782 0.203 1.145 0.363-2.793-5.297-7.575-9.29-13.41-11.16-3.067 2.66-5.008 6.585-5.008 10.962 0 0.832 0.065 1.628 0.198 2.41zM15.512 26.28c0.773 1.442 1.698 2.793 2.762 4.043 2.8-0.442 5.332-1.683 7.358-3.485-1.282 0.203-2.602 0.313-3.93 0.313-1.738 0-3.473-0.188-5.105-0.53-0.378-0.078-0.74-0.192-1.085-0.34zM12.485 22.983c-0.115-0.348-0.203-0.738-0.248-1.13-1.332 2.135-2.318 4.492-2.895 7.035 2.010 1.047 4.225 1.62 6.658 1.613 0.163 0 0.325-0.003 0.485-0.008-1.73-2.225-3.107-4.787-4-7.51zM14.412 16.82c-0.238-0.008-0.475-0.010-0.715-0.010-4.283-0.008-8.21 1.117-11.658 3.128 0.953 3.38 3.098 6.26 5.955 8.155 0.995-3.935 2.963-7.565 5.568-10.485 0.255-0.293 0.54-0.555 0.85-0.788z"></path></symbol>',
				'pp-bullhorn'      => '<symbol id="icon-pp-bullhorn" viewBox="0 0 36 32"><path fill="#ff5a30" style="fill: var(--color14, #ff5a30)" d="M5.933 28.196c-1.585 0.917-3.924-0.169-5.226-2.427-1.3-2.258-1.070-4.831 0.516-5.749l1.683-0.974c-0.867 0.609-0.915 2.97 0.385 5.228s3.356 3.378 4.325 2.948zM25.889 11.198c3.424 5.943 3.877 12.105 1.014 13.763-0.627 0.362-1.364 0.31-2.065 0.158-1.186-0.256-2.218-0.987-4.024-1.362-2.835-0.59-6.118-0.209-8.963 1.089 0.398 0.501 0.691 0.835 1.228 1.425 0.046-0.025 0.090-0.052 0.133-0.076 0.426-0.247 0.963-0.172 1.31 0.179 0.212 0.215 0.435 0.431 0.654 0.636 0.124 0.114 0.179 0.284 0.146 0.449-0.014 0.066-0.031 0.14-0.052 0.22-0.073 0.274-0.179 0.529-0.179 0.529 0.905 0.764 1.813 1.394 2.698 1.87 0.338 0.182 0.336 0.669-0.006 0.845-0.642 0.331-1.357 0.684-2.145 1.045-0.476 0.217-1.027 0.197-1.482-0.061-1.472-0.837-3.005-2.090-4.468-3.671-0.413-0.447-0.806-0.902-1.13-1.303-0.152-0.19-0.289-0.278-0.497-0.325-0.031-0.006-0.064-0.012-0.097-0.019 0 0-0.001 0-0.001 0s-0.001 0-0.001 0c-1.037-0.23-2.563-1.281-3.537-2.973-1.302-2.257-1.125-4.799-0.336-5.256l1.846-1.068c3.089-1.789 5.521-4.817 6.56-7.969 0.578-1.754 0.462-3.015 0.834-4.172 0.221-0.684 0.542-1.35 1.17-1.713 2.865-1.657 7.962 1.817 11.387 7.761zM6.452 20.436c1.57-0.967 5.33-3.424 7.695-5.971-0.504-1.153-0.903-2.542-0.903-2.542-1.203 2.129-4.060 5.083-7.134 7.038 0.050 0.511 0.194 1.032 0.343 1.474zM25.406 22.19c1.255-0.727 0.871-5.301-1.686-9.738s-6.367-7.029-7.572-6.331c-1.207 0.698-0.872 5.301 1.684 9.738s6.317 7.056 7.574 6.331zM21.239 17.439c0 0-0.84-0.973-1.744-2.542-0.905-1.57-1.318-2.787-1.318-2.787l1.971-0.443c0.266-0.060 0.543 0.043 0.702 0.265 0.356 0.498 0.701 1.028 1.023 1.588s0.608 1.12 0.861 1.677c0.114 0.25 0.065 0.543-0.12 0.745zM33.214 7.475c0.186 0.324 0.068 0.739-0.262 0.914l-3.381 1.808c-0.465 0.269-1.065 0.108-1.335-0.359-0.269-0.467-0.107-1.069 0.359-1.338l3.251-2.031c0.317-0.198 0.734-0.094 0.921 0.232zM35.851 16.355c0.038 0.373-0.239 0.703-0.611 0.729l-3.822 0.268c-0.535 0.055-1.017-0.337-1.073-0.873s0.336-1.020 0.871-1.075l3.795-0.524c0.371-0.052 0.709 0.215 0.747 0.587zM26.433 0.009l0.722 0.524c0.302 0.22 0.363 0.647 0.134 0.942l-2.349 3.033c-0.316 0.437-0.93 0.535-1.365 0.218-0.435-0.316-0.534-0.931-0.218-1.368l2.141-3.184c0.209-0.31 0.632-0.385 0.935-0.166z"></path></symbol>',
				'pp-castbox'       => '<symbol id="icon-pp-castbox" viewBox="0 0 32 32"><path fill="#f25c31" style="fill: var(--color15, #f25c31)" d="M2.15 16v-6.695c-0.005-1.020 0.425-1.75 1.3-2.27l11.17-6.615c0.93-0.555 1.8-0.56 2.735-0.005l11.215 6.64c0.86 0.505 1.285 1.25 1.285 2.235v13.395c0 0.985-0.41 1.735-1.27 2.245l-11.25 6.665c-0.9 0.53-1.755 0.55-2.655 0.010l-11.22-6.635c-0.895-0.53-1.325-1.275-1.315-2.315l0.005-6.655zM18.855 15.91l-0.005-3.070c-0.005-0.58-0.345-0.95-0.86-0.99-0.605-0.050-1.040 0.335-1.055 0.95l-0.010 2.015c-0.005 0.435-0.29 0.66-0.685 0.575-0.325-0.070-0.41-0.3-0.405-0.6v-2.67c0.005-0.38-0.1-0.7-0.435-0.905-0.685-0.42-1.48 0.010-1.5 0.83l-0.005 2.165c-0.005 0.395-0.185 0.585-0.525 0.6-0.35 0.010-0.575-0.2-0.595-0.6l-0.005-0.75c-0.020-0.415-0.23-0.7-0.615-0.83-0.65-0.23-1.28 0.205-1.29 0.905l-0.010 1.915c-0.005 0.345-0.21 0.535-0.525 0.545-0.325 0.010-0.53-0.165-0.575-0.505-0.015-0.1 0-0.205-0.015-0.3-0.060-0.5-0.475-0.88-0.95-0.87-0.48 0.005-0.935 0.385-0.945 0.87v3.025c0.015 0.5 0.465 0.87 0.945 0.87 0.485 0 0.885-0.365 0.95-0.87 0.015-0.115-0.030-0.26 0.025-0.345 0.11-0.17 0.245-0.38 0.415-0.435 0.15-0.045 0.405 0.050 0.525 0.17 0.11 0.105 0.145 0.33 0.15 0.5l0.010 2.975c0.010 0.725 0.755 1.17 1.395 0.845 0.365-0.185 0.515-0.485 0.51-0.905v-3.93c0-0.61 0.18-0.885 0.58-0.875 0.38 0.005 0.55 0.26 0.55 0.845v1.765c0.015 0.61 0.395 1 0.955 1.010 0.575 0.005 0.97-0.4 0.98-1.035l0.005-1.36c0.010-0.41 0.215-0.64 0.55-0.63 0.34 0.005 0.54 0.23 0.545 0.64l0.005 1.565c0.020 0.565 0.43 0.95 0.975 0.94 0.555-0.010 0.935-0.405 0.94-1.020v-3.025zM21.895 16.98v-1.71c-0.005-0.585-0.37-0.965-0.915-0.985-0.57-0.025-1 0.305-1.005 0.885l0.005 3.62c0.015 0.565 0.48 0.935 1.005 0.905 0.55-0.035 0.9-0.42 0.91-1.005v-1.71zM23.015 17.455l0.005 0.45c0.025 0.52 0.375 0.875 0.885 0.905 0.54 0.030 0.965-0.3 1.015-0.825 0.010-0.085 0.015-0.175 0.020-0.26 0.005-0.090 0.005-0.18 0.005-0.265 0-0.090 0-0.175-0.005-0.265-0.005-0.085-0.010-0.175-0.020-0.26-0.050-0.52-0.5-0.875-1.010-0.845-0.495 0.030-0.865 0.405-0.895 0.91-0.005 0.15 0 0.305 0 0.455v0z"></path><path fill="#fefefe" style="fill: var(--color16, #fefefe)" d="M18.855 15.915l-0.005 3.025c-0.005 0.615-0.38 1.010-0.94 1.015-0.545 0.010-0.95-0.375-0.975-0.94l-0.005-1.56c-0.005-0.41-0.205-0.64-0.545-0.645-0.335-0.005-0.54 0.22-0.55 0.635l-0.005 1.36c-0.010 0.63-0.405 1.040-0.98 1.035-0.56-0.005-0.94-0.4-0.955-1.010l0.005-1.765c0-0.59-0.165-0.845-0.55-0.85-0.4-0.005-0.58 0.27-0.58 0.88l0.005 3.93c0 0.415-0.15 0.72-0.515 0.905-0.64 0.32-1.385-0.125-1.395-0.85l-0.010-2.97c-0.005-0.175-0.035-0.395-0.15-0.505-0.12-0.12-0.375-0.21-0.525-0.165-0.17 0.050-0.305 0.265-0.415 0.435-0.050 0.085-0.010 0.23-0.025 0.345-0.065 0.5-0.465 0.87-0.95 0.87-0.475 0-0.93-0.38-0.945-0.875v-3.020c0.015-0.49 0.465-0.87 0.945-0.875 0.475-0.005 0.89 0.375 0.95 0.87 0.015 0.1 0 0.205 0.015 0.3 0.050 0.34 0.25 0.52 0.575 0.51 0.315-0.010 0.52-0.2 0.53-0.55l0.005-1.915c0.015-0.7 0.64-1.13 1.29-0.9 0.385 0.135 0.6 0.41 0.615 0.83l0.005 0.75c0.020 0.395 0.24 0.605 0.595 0.6 0.34-0.015 0.52-0.205 0.525-0.6l0.005-2.17c0.025-0.82 0.815-1.25 1.5-0.83 0.335 0.21 0.44 0.53 0.435 0.905v2.67c0 0.3 0.085 0.53 0.405 0.605 0.395 0.085 0.68-0.14 0.685-0.575l0.010-2.015c0.015-0.62 0.455-1 1.055-0.95 0.515 0.040 0.855 0.41 0.86 0.985l0.005 3.075zM21.895 16.985v1.71c-0.010 0.585-0.365 0.97-0.91 1-0.53 0.030-0.99-0.335-1.005-0.9l-0.005-3.62c0.010-0.58 0.435-0.91 1.005-0.89 0.55 0.025 0.905 0.405 0.915 0.985v1.715zM23.015 17.46l0.005-0.45c0.025-0.51 0.4-0.88 0.89-0.91 0.515-0.035 0.96 0.32 1.010 0.84 0.035 0.345 0.035 0.7 0.005 1.050-0.055 0.53-0.48 0.855-1.020 0.825-0.505-0.025-0.86-0.385-0.885-0.9-0.010-0.15 0-0.305 0-0.455h-0.005z"></path></symbol>',
				'pp-castro'        => '<symbol id="icon-pp-castro" viewBox="0 0 32 32"><defs><linearGradient id="a"><stop offset="0" stop-color="#00CCBF"/><stop offset="1" stop-color="#00B265"/></linearGradient></defs><path fill="url(#a)" d="M11.157 0h9.687c3.212 0 4.819 0 6.548 0.547 1.888 0.687 3.375 2.174 4.062 4.062 0.547 1.729 0.547 3.335 0.547 6.548v9.687c0 3.212 0 4.819-0.547 6.548-0.687 1.888-2.174 3.375-4.062 4.062-1.729 0.547-3.335 0.547-6.548 0.547h-9.687c-3.213 0-4.819 0-6.548-0.547-1.888-0.687-3.375-2.174-4.062-4.062-0.547-1.729-0.547-3.335-0.547-6.548v-9.687c0-3.213 0-4.819 0.547-6.548 0.687-1.888 2.174-3.375 4.062-4.062 1.729-0.547 3.335-0.547 6.548-0.547v0z"></path><path fill="#000" style="fill: var(--color7, #000)" d="M16.016 2.094c7.689 0 13.922 6.227 13.922 13.908s-6.233 13.908-13.922 13.908-13.922-6.227-13.922-13.908c0-7.681 6.233-13.908 13.922-13.908z"></path><path fill="#fff" style="fill: var(--color8, #fff)" d="M21.809 23.934l-2.334-3.235c1.466-1.072 2.418-2.804 2.418-4.758 0-3.254-2.641-5.892-5.898-5.892s-5.898 2.638-5.898 5.892c0 1.963 0.961 3.701 2.438 4.772l-2.336 3.237c-2.483-1.796-4.099-4.715-4.099-8.011 0-5.458 4.429-9.883 9.893-9.883s9.893 4.425 9.893 9.883c0 3.286-1.606 6.197-4.076 7.994zM16.014 13.554c1.314 0 2.38 1.065 2.38 2.378s-1.066 2.378-2.38 2.378-2.38-1.064-2.38-2.378c0-1.313 1.066-2.378 2.38-2.378z"></path></symbol>',
				'pp-downcast'      => '<symbol id="icon-pp-downcast" viewBox="0 0 32 32"><path fill="#e10000" style="fill: var(--color17, #e10000)" d="M16 31.678c-8.671 0-15.678-7.007-15.678-15.678s7.007-15.678 15.678-15.678c8.671 0 15.678 7.007 15.678 15.678s-7.007 15.678-15.678 15.678z"></path><path fill="#fff" style="fill: var(--color8, #fff)" d="M26.303 16.256c0 1.056-0.224 2.368-0.576 3.68-0.256 0.928-0.608 1.76-1.024 2.624-0.64 1.312-1.504 2.496-2.528 3.552-0.064 0.064-0.128 0.128-0.192 0.192-0.16 0.128-0.32 0.128-0.48-0.032-0.352-0.352-0.704-0.704-1.088-1.056-0.16-0.16-0.32-0.32-0.48-0.448-0.192-0.16-0.192-0.352 0-0.512 0.576-0.576 1.088-1.216 1.536-1.92s0.8-1.44 1.088-2.208c0.32-0.896 0.576-1.792 0.672-2.72 0.256-1.664 0.128-3.328-0.384-4.928-0.512-1.632-1.344-3.104-2.496-4.384-0.128-0.128-0.256-0.256-0.352-0.384-0.064-0.096-0.128-0.192-0.096-0.288 0.032-0.064 0.064-0.128 0.128-0.192 0.48-0.448 0.992-0.992 1.568-1.504 0.192-0.16 0.352-0.16 0.544 0 0.32 0.32 0.64 0.672 0.96 1.056 1.536 1.92 2.496 4.096 2.976 6.463 0.16 0.896 0.224 1.792 0.224 3.008z"></path><path fill="#fff" style="fill: var(--color8, #fff)" d="M20.16 16c0 1.44-0.352 2.848-0.992 4.192-0.416 0.896-0.992 1.728-1.696 2.432-0.032 0.032-0.032 0.032-0.064 0.064-0.16 0.16-0.32 0.16-0.48 0-0.544-0.512-1.056-1.024-1.6-1.568-0.128-0.128-0.16-0.288-0.032-0.448 0.192-0.224 0.384-0.448 0.576-0.672 0.704-0.992 1.12-2.080 1.28-3.264 0.128-0.928 0.064-1.824-0.224-2.72-0.288-1.024-0.832-1.92-1.568-2.72-0.192-0.192-0.16-0.352 0-0.544 0.512-0.48 1.024-0.992 1.536-1.472 0.032-0.032 0.064-0.064 0.096-0.064 0.16-0.096 0.288-0.096 0.416 0.032 0.256 0.224 0.48 0.512 0.672 0.768 0.864 1.088 1.408 2.336 1.76 3.648 0.224 0.768 0.32 1.536 0.32 2.336z"></path><path fill="#fff" style="fill: var(--color8, #fff)" d="M13.44 18.112c-0.512 1.344-1.536 2.176-2.976 2.496-0.448 0.096-0.896 0.192-1.408 0.192-1.088 0-1.664 0-2.752 0-0.16 0-0.224-0.064-0.224-0.192 0-1.408 0-3.104 0-4.544s0-2.944 0-4.384c0-0.224 0.064-0.288 0.256-0.288 1.152 0 1.856 0 3.008 0 0.8 0 1.6 0.16 2.304 0.576 0.832 0.48 1.44 1.184 1.792 2.080 0.224 0.512 0.32 1.056 0.352 1.6 0.064 0.832-0.032 1.664-0.352 2.464zM7.905 18.464c0 0.192 0.096 0.256 0.32 0.256 0.288 0 0.608 0 0.928 0s0.64-0.064 0.96-0.192c0.544-0.224 0.896-0.64 1.12-1.184 0.288-0.768 0.288-1.696 0.064-2.496-0.288-0.96-0.992-1.728-2.016-1.728-0.416 0-0.704 0-1.088 0-0.192 0-0.288 0.032-0.288 0.224 0 0.8 0 4.32 0 5.119z"></path></symbol>',
				'pp-iheartradio'   => '<symbol id="icon-pp-iheartradio" viewBox="0 0 43 32"><path fill="#c6002b" style="fill: var(--color18, #c6002b)" d="M9.931 20.884c-3.445-3.051-5.184-6.229-5.184-9.248s1.696-6.176 3.936-7.584c1.099-0.469 1.557 0.875 0.853 1.344-1.781 1.12-3.189 3.744-3.189 6.261s1.568 5.291 4.651 8.021c0.747 0.928-0.405 1.76-1.056 1.195zM15.105 18.164c-2.336-1.312-3.904-3.872-3.904-6.336s1.333-4.661 3.445-5.76c1.12-0.395 1.493 0.992 0.747 1.408-1.6 0.832-2.592 2.485-2.592 4.288 0.011 1.963 1.248 3.968 3.093 5.003 0.928 0.757-0.064 1.781-0.789 1.408zM21.515 15.828c-1.909 0-3.467-1.547-3.467-3.456s1.557-3.467 3.467-3.467v-3.275c-2.069-3.083-5.632-5.205-9.632-5.205-6.357 0-11.509 5.152-11.509 11.509 0 4.032 2.56 6.944 5.227 9.632l10.443 9.877c0.672 0.416 1.44 0.139 1.451-0.619v-7.691c0-2.229 1.803-4.032 4.032-4.032z"></path><path fill="#c6002b" style="fill: var(--color18, #c6002b)" d="M32.886 20.884c3.445-3.051 5.184-6.229 5.184-9.248s-1.696-6.176-3.936-7.584c-1.099-0.469-1.557 0.875-0.853 1.344 1.781 1.12 3.189 3.744 3.189 6.261s-1.568 5.291-4.651 8.021c-0.747 0.928 0.405 1.76 1.056 1.195zM27.713 18.164c2.336-1.312 3.904-3.872 3.904-6.336s-1.333-4.661-3.445-5.76c-1.12-0.395-1.493 0.992-0.747 1.408 1.6 0.832 2.592 2.485 2.592 4.288-0.011 1.963-1.248 3.968-3.093 5.003-0.928 0.757 0.064 1.781 0.789 1.408zM21.302 15.828c1.909 0 3.467-1.547 3.467-3.456s-1.557-3.467-3.467-3.467v-3.275c2.069-3.083 5.632-5.205 9.632-5.205 6.357 0 11.509 5.152 11.509 11.509 0 4.032-2.56 6.944-5.227 9.632l-10.443 9.877c-0.672 0.416-1.44 0.139-1.451-0.619v-7.691c0-2.229-1.803-4.032-4.032-4.032z"></path></symbol>',
				'pp-overcast'      => '<symbol id="icon-pp-overcast" viewBox="0 0 32 32"><path fill="#f57e20" style="fill: var(--color19, #f57e20)" d="M7.906 24.094c-2.072-2.071-3.353-4.933-3.353-8.094s1.281-6.023 3.353-8.094l1.018 1.018c-1.811 1.811-2.931 4.313-2.931 7.076s1.12 5.265 2.931 7.076zM11.037 20.963c-1.27-1.27-2.056-3.025-2.056-4.963s0.786-3.693 2.056-4.963l1.018 1.018c-1.010 1.010-1.634 2.404-1.634 3.945s0.625 2.935 1.634 3.945zM19.945 19.945c1.010-1.010 1.634-2.404 1.634-3.945s-0.625-2.935-1.634-3.945l1.018-1.018c1.27 1.27 2.056 3.025 2.056 4.963s-0.786 3.693-2.056 4.963zM24.094 24.094l-1.018-1.018c1.811-1.811 2.931-4.313 2.931-7.076s-1.12-5.265-2.931-7.076l1.018-1.018c2.072 2.071 3.353 4.933 3.353 8.094s-1.281 6.023-3.353 8.094zM16 1.459c-8.031 0-14.541 6.51-14.541 14.541 0 6.423 4.165 11.874 9.942 13.798l3.734-11.203c-1.087-0.362-1.871-1.387-1.871-2.595 0-1.511 1.225-2.736 2.736-2.736s2.736 1.225 2.736 2.736c0 1.208-0.784 2.233-1.871 2.595l3.734 11.203c5.777-1.925 9.942-7.375 9.942-13.798 0-8.031-6.51-14.541-14.541-14.541zM17.789 30.431c-0.647-0.503-1.274-1.056-1.873-1.661-0.579 0.599-1.183 1.149-1.805 1.649 0.619 0.080 1.249 0.122 1.89 0.122 0.605 0 1.202-0.037 1.789-0.11zM16.815 27.776c0.798 0.898 1.654 1.691 2.547 2.374 0.060-0.014 0.119-0.029 0.178-0.044l-1.545-3.862c-0.376 0.536-0.771 1.047-1.181 1.532zM15.979 26.77c0.499-0.643 0.968-1.331 1.403-2.060l-1.381-3.453-1.398 3.496c0.427 0.714 0.888 1.387 1.377 2.018zM12.605 30.142c0.862-0.66 1.69-1.421 2.463-2.282-0.395-0.452-0.775-0.929-1.138-1.43l-1.471 3.676c0.048 0.012 0.097 0.024 0.146 0.035zM16 31.982c-8.827 0-15.982-7.155-15.982-15.982s7.155-15.982 15.982-15.982c8.827 0 15.982 7.155 15.982 15.982s-7.155 15.982-15.982 15.982z"></path><path fill="#fff" style="fill: var(--color8, #fff)" d="M7.906 24.094l0.013-0.013c-2.068-2.068-3.348-4.925-3.348-8.081s1.279-6.013 3.348-8.081l-0.013-0.013-0.013 0.013 1.018 1.018 0.013-0.013-0.013-0.013c-1.814 1.814-2.936 4.321-2.936 7.089s1.122 5.274 2.936 7.089l0.013-0.013-0.013-0.013-1.018 1.018 0.026 0.026 1.031-1.031-0.013-0.013c-1.808-1.808-2.926-4.304-2.926-7.063s1.118-5.255 2.926-7.063l0.013-0.013-1.043-1.043-0.013 0.013c-2.075 2.075-3.358 4.941-3.358 8.107s1.283 6.032 3.358 8.107l0.013 0.013 0.013-0.013zM11.037 20.963l0.013-0.013c-1.267-1.267-2.051-3.017-2.051-4.95s0.784-3.684 2.051-4.95l-0.013-0.013-0.013 0.013 1.018 1.018 0.013-0.013-0.013-0.013c-1.013 1.013-1.639 2.412-1.639 3.958s0.626 2.945 1.639 3.958l0.013-0.013-0.013-0.013-1.018 1.018 0.026 0.026 1.031-1.031-0.013-0.013c-1.007-1.006-1.629-2.397-1.629-3.932s0.622-2.926 1.629-3.932l0.013-0.013-1.043-1.044-0.013 0.013c-1.273 1.274-2.061 3.033-2.061 4.976s0.788 3.702 2.061 4.976l0.013 0.013 0.013-0.013zM19.945 19.945l0.013 0.013c1.013-1.013 1.639-2.412 1.639-3.958s-0.626-2.945-1.639-3.958l-0.013 0.013 0.013 0.013 1.018-1.018-0.013-0.013-0.013 0.013c1.267 1.267 2.051 3.017 2.051 4.95s-0.784 3.684-2.051 4.95l0.013 0.013 0.013-0.013-1.018-1.018-0.026 0.026 1.031 1.031 0.013-0.013c1.273-1.274 2.061-3.033 2.061-4.976s-0.788-3.702-2.061-4.976l-0.013-0.013-1.044 1.044 0.013 0.013c1.006 1.006 1.629 2.397 1.629 3.932s-0.622 2.926-1.629 3.932l-0.013 0.013 0.013 0.013zM24.094 24.094l0.013-0.013-1.018-1.018-0.013 0.013 0.013 0.013c1.814-1.814 2.936-4.321 2.936-7.089s-1.122-5.274-2.936-7.089l-0.013 0.013 0.013 0.013 1.018-1.018-0.013-0.013-0.013 0.013c2.068 2.068 3.347 4.925 3.347 8.081s-1.279 6.013-3.347 8.081l0.026 0.026c2.075-2.075 3.358-4.941 3.358-8.107s-1.283-6.032-3.358-8.107l-0.013-0.013-1.044 1.044 0.013 0.013c1.808 1.808 2.926 4.304 2.926 7.063s-1.118 5.255-2.926 7.063l-0.013 0.013 1.044 1.044 0.013-0.013zM16 1.459v-0.018c-8.041 0-14.559 6.519-14.559 14.559 0 6.431 4.171 11.888 9.954 13.815l0.017 0.005 3.746-11.237-0.017-0.006c-1.080-0.36-1.858-1.378-1.858-2.578 0-0.75 0.304-1.43 0.796-1.922s1.171-0.796 1.922-0.796c0.75 0 1.43 0.304 1.922 0.796s0.796 1.171 0.796 1.922c0 1.2-0.778 2.218-1.858 2.578l-0.017 0.006 3.746 11.237 0.017-0.005c5.784-1.927 9.954-7.384 9.954-13.815 0-8.041-6.518-14.559-14.559-14.559v0.036c4.011 0 7.641 1.626 10.27 4.254s4.254 6.259 4.254 10.27c0 6.415-4.16 11.859-9.929 13.781l0.005 0.017 0.017-0.006-3.734-11.203-0.017 0.006 0.006 0.017c1.094-0.364 1.883-1.396 1.883-2.612 0-1.521-1.233-2.753-2.754-2.754s-2.753 1.233-2.754 2.754c0 1.216 0.789 2.248 1.883 2.612l0.006-0.017-0.017-0.006-3.734 11.203 0.017 0.006 0.005-0.017c-5.769-1.922-9.929-7.366-9.929-13.781 0-4.011 1.626-7.641 4.254-10.27s6.259-4.254 10.27-4.254zM17.789 30.431l0.011-0.014c-0.647-0.502-1.273-1.056-1.871-1.66l-0.013-0.013-0.013 0.013c-0.578 0.599-1.182 1.148-1.804 1.647l-0.033 0.026 0.042 0.005c0.62 0.080 1.251 0.122 1.892 0.122 0.606 0 1.204-0.038 1.791-0.11l0.043-0.005-0.034-0.027-0.011 0.014-0.002-0.018c-0.585 0.072-1.181 0.11-1.786 0.11-0.64 0-1.27-0.042-1.887-0.122l-0.002 0.018 0.011 0.014c0.623-0.5 1.228-1.051 1.807-1.65l-0.013-0.012-0.013 0.013c0.599 0.605 1.227 1.159 1.874 1.663l0.011-0.014-0.002-0.018zM16.815 27.776l-0.014 0.012c0.798 0.898 1.655 1.692 2.55 2.375l0.007 0.005 0.008-0.002c0.060-0.014 0.119-0.029 0.178-0.044l0.020-0.005-1.564-3.911-0.019 0.027c-0.376 0.536-0.77 1.046-1.18 1.53l-0.010 0.012 0.010 0.012 0.014-0.012 0.014 0.012c0.411-0.485 0.805-0.997 1.182-1.533l-0.015-0.010-0.017 0.007 1.545 3.861 0.017-0.006-0.004-0.017c-0.059 0.015-0.119 0.029-0.178 0.043l0.004 0.018 0.011-0.014c-0.893-0.682-1.748-1.474-2.545-2.371l-0.013 0.012 0.014 0.012zM15.979 26.77l0.014 0.011c0.499-0.643 0.969-1.332 1.404-2.062l0.004-0.008-1.401-3.503-1.418 3.546 0.005 0.008c0.428 0.714 0.889 1.389 1.378 2.020l0.014 0.018 0.014-0.018-0.014-0.011 0.014-0.011c-0.489-0.63-0.949-1.303-1.376-2.016l-0.015 0.009 0.017 0.007 1.382-3.454 1.365 3.412 0.017-0.006-0.016-0.009c-0.434 0.729-0.903 1.416-1.401 2.059l0.014 0.011 0.014-0.011zM12.605 30.142l0.011 0.014c0.863-0.66 1.691-1.423 2.465-2.284l0.011-0.012-0.011-0.012c-0.395-0.452-0.775-0.929-1.137-1.429l-0.019-0.027-1.49 3.726 0.020 0.005c0.049 0.012 0.097 0.024 0.146 0.036l0.008 0.002 0.007-0.005-0.011-0.014 0.004-0.017c-0.049-0.012-0.097-0.023-0.145-0.035l-0.004 0.017 0.017 0.006 1.471-3.676-0.017-0.007-0.015 0.011c0.363 0.501 0.744 0.979 1.139 1.431l0.013-0.012-0.013-0.012c-0.773 0.859-1.599 1.621-2.461 2.279l0.011 0.014 0.004-0.017zM16 31.982v-0.018c-4.408 0-8.399-1.787-11.288-4.676s-4.676-6.88-4.676-11.288 1.787-8.399 4.676-11.288c2.889-2.889 6.88-4.676 11.288-4.676s8.399 1.787 11.288 4.676c2.889 2.889 4.676 6.88 4.676 11.288s-1.787 8.399-4.676 11.288c-2.889 2.889-6.88 4.676-11.288 4.676v0.036c8.837 0 16-7.163 16-16s-7.163-16-16-16c-8.837 0-16 7.163-16 16s7.163 16 16 16v-0.018z"></path></symbol>',
				'pp-playerfm'      => '<symbol id="icon-pp-playerfm" viewBox="0 0 32 32"><path fill="#c8122a" style="fill: var(--color20, #c8122a)" d="M15.275 0.185c4.97-0.27 9.965 2.020 13.050 5.91 2.945 3.625 4.115 8.59 3.13 13.155-1.050 5.14-4.85 9.6-9.755 11.46-2.225 0.885-4.66 1.21-7.045 1.010-4.375-0.325-8.53-2.63-11.195-6.105-1.37-1.78-2.35-3.855-2.865-6.045-0.995-4.21-0.175-8.82 2.27-12.405 2.72-4.115 7.475-6.805 12.41-6.98zM18.835 3.975c-0.825 0.14-1.020 1.425-0.235 1.755 3.45 1.030 6.31 3.805 7.44 7.22 0.11 0.47 0.46 0.96 0.985 0.97 0.61 0.060 1.13-0.575 0.985-1.165-1.16-3.93-4.285-7.21-8.145-8.585-0.33-0.11-0.675-0.275-1.030-0.195zM13.66 5.8c-2.76 0.565-5.21 2.395-6.66 4.795-1.81 2.955-2 6.83-0.485 9.945 1.54 3.3 4.91 5.66 8.545 5.94 2.25 0.215 4.57-0.335 6.49-1.535 2.315-1.44 4.050-3.805 4.66-6.465 0.21-0.995-0.47-2.045-1.415-2.37-0.91-0.35-2.025-0.005-2.575 0.8-0.395 0.535-0.465 1.225-0.78 1.805-0.915 1.885-2.865 3.2-4.955 3.355-2.625 0.25-5.275-1.435-6.175-3.91-0.67-1.755-0.48-3.815 0.54-5.395 0.805-1.3 2.13-2.265 3.615-2.64 1.085-0.28 1.795-1.485 1.545-2.57-0.2-1.060-1.27-1.885-2.35-1.755zM18.365 7.040c-0.42 0.050-0.775 0.4-0.83 0.82-0.070 0.455 0.235 0.915 0.665 1.060 2.195 0.82 3.98 2.635 4.79 4.83 0.14 0.475 0.68 0.8 1.165 0.66 0.51-0.12 0.85-0.71 0.66-1.21-0.93-2.685-3.075-4.925-5.715-5.98-0.24-0.080-0.475-0.215-0.735-0.18zM17.76 10.27c-0.825 0.095-1.075 1.365-0.32 1.735 1.12 0.53 1.985 1.48 2.525 2.585 0.515 0.835 1.96 0.225 1.705-0.735-0.51-1.37-1.585-2.5-2.82-3.26-0.325-0.195-0.695-0.39-1.090-0.325z"></path><path fill="#fff" style="fill: var(--color8, #fff)" d="M18.835 3.975c0.355-0.080 0.7 0.085 1.030 0.195 3.86 1.375 6.985 4.655 8.145 8.585 0.145 0.59-0.375 1.225-0.985 1.165-0.525-0.010-0.875-0.5-0.985-0.97-1.13-3.415-3.99-6.19-7.44-7.22-0.785-0.33-0.59-1.615 0.235-1.755z"></path><path fill="#fff" style="fill: var(--color8, #fff)" d="M13.66 5.8c1.080-0.13 2.15 0.695 2.35 1.755 0.25 1.085-0.46 2.29-1.545 2.57-1.485 0.375-2.81 1.34-3.615 2.64-1.020 1.58-1.21 3.64-0.54 5.395 0.9 2.475 3.55 4.16 6.175 3.91 2.090-0.155 4.040-1.47 4.955-3.355 0.315-0.58 0.385-1.27 0.78-1.805 0.55-0.805 1.665-1.15 2.575-0.8 0.945 0.325 1.625 1.375 1.415 2.37-0.61 2.66-2.345 5.025-4.66 6.465-1.92 1.2-4.24 1.75-6.49 1.535-3.635-0.28-7.005-2.64-8.545-5.94-1.515-3.115-1.325-6.99 0.485-9.945 1.45-2.4 3.9-4.23 6.66-4.795z"></path><path fill="#fff" style="fill: var(--color8, #fff)" d="M18.365 7.040c0.26-0.035 0.495 0.1 0.735 0.18 2.64 1.055 4.785 3.295 5.715 5.98 0.19 0.5-0.15 1.090-0.66 1.21-0.485 0.14-1.025-0.185-1.165-0.66-0.81-2.195-2.595-4.010-4.79-4.83-0.43-0.145-0.735-0.605-0.665-1.060 0.055-0.42 0.41-0.77 0.83-0.82z"></path><path fill="#fff" style="fill: var(--color8, #fff)" d="M17.76 10.27c0.395-0.065 0.765 0.13 1.090 0.325 1.235 0.76 2.31 1.89 2.82 3.26 0.255 0.96-1.19 1.57-1.705 0.735-0.54-1.105-1.405-2.055-2.525-2.585-0.755-0.37-0.505-1.64 0.32-1.735z"></path></symbol>',
				'pp-pocketcasts'   => '<symbol id="icon-pp-pocketcasts" viewBox="0 0 32 32"><path fill="#f44336" style="fill: var(--color21, #f44336)" d="M3.365 28.635c-2.455-2.455-3.365-6.455-3.365-12.635s0.91-10.18 3.365-12.635c2.455-2.455 6.455-3.365 12.635-3.365s10.18 0.91 12.635 3.365c2.455 2.455 3.365 6.455 3.365 12.635s-0.91 10.18-3.365 12.635c-2.455 2.455-6.455 3.365-12.635 3.365s-10.18-0.91-12.635-3.365z"></path><path fill="#fff" style="fill: var(--color8, #fff)" d="M15.71 24.315c0.040 0 0.075 0.010 0.11 0.025s0.065 0.040 0.095 0.065c0.025 0.025 0.045 0.060 0.060 0.095s0.020 0.070 0.020 0.11v2.42c0 0.040-0.010 0.080-0.025 0.115-0.015 0.040-0.035 0.070-0.065 0.1-0.030 0.025-0.060 0.045-0.1 0.060-0.035 0.015-0.075 0.025-0.115 0.020-4.525-0.12-8.55-2.925-10.225-7.135-1.675-4.205-0.69-9.005 2.515-12.21 3.2-3.205 8.005-4.19 12.21-2.515 4.21 1.675 7.015 5.7 7.135 10.225 0 0.040-0.005 0.080-0.020 0.115-0.015 0.040-0.035 0.070-0.060 0.1-0.030 0.030-0.065 0.050-0.1 0.065s-0.075 0.025-0.115 0.025h-2.42c-0.040 0-0.075-0.005-0.11-0.020s-0.070-0.035-0.095-0.060c-0.025-0.030-0.050-0.060-0.065-0.095s-0.025-0.070-0.025-0.11c-0.115-3.31-2.18-6.24-5.265-7.46-3.085-1.215-6.595-0.485-8.94 1.86s-3.075 5.855-1.86 8.94c1.22 3.085 4.15 5.15 7.46 5.265zM15.995 9.44c0.835 0 1.66 0.16 2.435 0.47s1.485 0.765 2.090 1.345c0.6 0.575 1.090 1.26 1.435 2.020 0.35 0.76 0.545 1.58 0.585 2.41 0.005 0.040-0.005 0.080-0.015 0.12-0.015 0.035-0.040 0.070-0.065 0.1-0.030 0.030-0.060 0.050-0.1 0.065-0.035 0.015-0.075 0.025-0.115 0.025h-1.92c-0.040 0-0.075-0.010-0.11-0.020-0.030-0.015-0.065-0.035-0.090-0.060-0.030-0.025-0.050-0.055-0.065-0.090-0.015-0.030-0.025-0.065-0.030-0.105-0.105-1.58-1.13-2.955-2.615-3.51-1.485-0.56-3.16-0.195-4.28 0.925s-1.485 2.795-0.925 4.28c0.555 1.485 1.93 2.51 3.51 2.615 0.040 0.005 0.075 0.015 0.105 0.030 0.035 0.015 0.065 0.040 0.090 0.065s0.045 0.060 0.060 0.090c0.010 0.035 0.020 0.070 0.020 0.11v1.92c0 0.040-0.010 0.080-0.025 0.115-0.015 0.040-0.035 0.070-0.065 0.1-0.030 0.025-0.065 0.050-0.1 0.065-0.040 0.010-0.080 0.020-0.12 0.015-1.705-0.080-3.315-0.825-4.485-2.080-1.165-1.25-1.795-2.91-1.755-4.62s0.75-3.335 1.975-4.53c1.22-1.2 2.865-1.865 4.575-1.87z"></path></symbol>',
				'pp-podbean'       => '<symbol id="icon-pp-podbean" viewBox="0 0 32 32"><path fill="#fff" style="fill: var(--color8, #fff)" d="M15.8 19.425c0.805-0.080 1.6 0.285 2.15 0.865 0.175 0.22 0.33 0.465 0.455 0.725 0.195 0.445 0.205 0.945 0.175 1.425-0.125 0.685-0.5 1.345-1.095 1.73-0.35 0.255-0.765 0.42-1.2 0.445-0.305 0.020-0.615 0.025-0.91-0.035-0.37-0.090-0.71-0.275-1.020-0.49-0.365-0.3-0.645-0.7-0.82-1.14-0.265-0.715-0.22-1.56 0.19-2.215 0.13-0.205 0.255-0.42 0.44-0.575 0.245-0.215 0.51-0.41 0.805-0.545 0.27-0.095 0.55-0.155 0.83-0.19z"></path><path fill="#252027" style="fill: var(--color22, #252027)" d="M2.225 0.185c0.535-0.040 1.070-0.020 1.605-0.020 8.225 0 16.445 0 24.67 0 0.4 0 0.8-0.015 1.2 0 0.365 0.040 0.705 0.195 1.020 0.38 0.53 0.38 0.97 0.95 1.065 1.605 0.030 0.465 0.005 0.93 0.010 1.39 0 8.63 0 17.26 0 25.885 0 0.31-0.025 0.625-0.165 0.905-0.15 0.36-0.38 0.69-0.685 0.94-0.385 0.285-0.825 0.54-1.32 0.525-9.1 0-18.2 0-27.3 0-0.795-0.035-1.505-0.58-1.875-1.265-0.15-0.24-0.21-0.525-0.26-0.805-0.050-0.555-0.025-1.12-0.025-1.68 0-8.15 0-16.305 0-24.46 0-0.43-0.015-0.86 0.005-1.29 0.11-1.055 1.005-1.97 2.055-2.11zM15.715 5.12c-1.495 0.030-2.99 0.27-4.42 0.715-0.975 0.32-1.925 0.72-2.825 1.205-1.32 0.755-2.55 1.665-3.61 2.755-0.69 0.725-1.34 1.5-1.89 2.345-0.080 0.14-0.175 0.275-0.23 0.425-0.17 0.45-0.090 0.97 0.16 1.375 0.19 0.355 0.55 0.59 0.93 0.705 0.24 0.065 0.495 0.030 0.745 0.010 0.21-0.030 0.4-0.14 0.58-0.25 0.2-0.135 0.345-0.34 0.485-0.53 0.225-0.345 0.47-0.67 0.72-0.995 1.445-1.745 3.345-3.14 5.5-3.87 0.6-0.235 1.23-0.375 1.86-0.51 0.785-0.155 1.585-0.2 2.38-0.19 0.8-0.005 1.595 0.1 2.38 0.24 0.59 0.115 1.18 0.27 1.745 0.495 0.245 0.095 0.49 0.18 0.735 0.285 0.65 0.285 1.27 0.64 1.865 1.025 1.22 0.79 2.27 1.82 3.135 2.98 0.25 0.325 0.43 0.7 0.735 0.975 0.325 0.265 0.75 0.425 1.17 0.36 0.33 0.005 0.63-0.155 0.89-0.345 0.365-0.315 0.59-0.79 0.575-1.275-0.025-0.22-0.055-0.445-0.155-0.645-0.47-0.785-1.045-1.5-1.65-2.18-0.265-0.28-0.545-0.545-0.815-0.82-0.44-0.445-0.93-0.84-1.435-1.21-0.87-0.68-1.835-1.23-2.835-1.69-0.85-0.375-1.72-0.71-2.63-0.92-0.61-0.145-1.225-0.285-1.85-0.35-0.745-0.095-1.495-0.15-2.245-0.115zM15.87 10.86c-1.13 0.035-2.265 0.215-3.32 0.63-0.645 0.245-1.265 0.555-1.84 0.93-0.585 0.395-1.155 0.825-1.65 1.33-0.42 0.44-0.81 0.91-1.15 1.415-0.13 0.195-0.255 0.4-0.31 0.63-0.095 0.265-0.045 0.54 0.010 0.805 0.12 0.515 0.545 0.92 1.035 1.095 0.545 0.19 1.185 0.035 1.6-0.36 0.265-0.3 0.465-0.645 0.725-0.945 0.365-0.435 0.79-0.82 1.25-1.155 1.055-0.715 2.31-1.18 3.595-1.175 0.505 0 1.015 0.020 1.51 0.125 1.185 0.23 2.28 0.835 3.165 1.645 0.35 0.345 0.675 0.72 0.95 1.13 0.115 0.175 0.255 0.34 0.42 0.475 0.445 0.36 1.090 0.445 1.615 0.21 0.385-0.185 0.73-0.495 0.86-0.905 0.14-0.44 0.12-0.94-0.115-1.34-0.38-0.645-0.87-1.215-1.4-1.735-1.815-1.82-4.385-2.855-6.95-2.805zM15.76 17.060c-0.925 0.045-1.84 0.335-2.595 0.875-0.52 0.335-0.95 0.8-1.3 1.31-0.295 0.405-0.5 0.87-0.65 1.345-0.335 1.025-0.26 2.15 0.090 3.16 0.19 0.485 0.445 0.945 0.76 1.36 0.415 0.5 0.9 0.945 1.47 1.265 0.545 0.295 1.145 0.52 1.765 0.6 0.555 0.085 1.12 0.035 1.67-0.060 0.88-0.16 1.695-0.595 2.36-1.185 0.82-0.735 1.365-1.75 1.58-2.825 0.060-0.425 0.055-0.855 0.035-1.28-0.050-0.885-0.385-1.74-0.885-2.47-0.955-1.365-2.635-2.205-4.3-2.095z"></path><path fill="#8dc742" style="fill: var(--color23, #8dc742)" d="M15.715 5.12c0.75-0.035 1.5 0.020 2.245 0.115 0.625 0.065 1.24 0.205 1.85 0.35 0.91 0.21 1.78 0.545 2.63 0.92 1 0.46 1.965 1.010 2.835 1.69 0.505 0.37 0.995 0.765 1.435 1.21 0.27 0.275 0.55 0.54 0.815 0.82 0.605 0.68 1.18 1.395 1.65 2.18 0.1 0.2 0.13 0.425 0.155 0.645 0.015 0.485-0.21 0.96-0.575 1.275-0.26 0.19-0.56 0.35-0.89 0.345-0.42 0.065-0.845-0.095-1.17-0.36-0.305-0.275-0.485-0.65-0.735-0.975-0.865-1.16-1.915-2.19-3.135-2.98-0.595-0.385-1.215-0.74-1.865-1.025-0.245-0.105-0.49-0.19-0.735-0.285-0.565-0.225-1.155-0.38-1.745-0.495-0.785-0.14-1.58-0.245-2.38-0.24-0.795-0.010-1.595 0.035-2.38 0.19-0.63 0.135-1.26 0.275-1.86 0.51-2.155 0.73-4.055 2.125-5.5 3.87-0.25 0.325-0.495 0.65-0.72 0.995-0.14 0.19-0.285 0.395-0.485 0.53-0.18 0.11-0.37 0.22-0.58 0.25-0.25 0.020-0.505 0.055-0.745-0.010-0.38-0.115-0.74-0.35-0.93-0.705-0.25-0.405-0.33-0.925-0.16-1.375 0.055-0.15 0.15-0.285 0.23-0.425 0.55-0.845 1.2-1.62 1.89-2.345 1.060-1.090 2.29-2 3.61-2.755 0.9-0.485 1.85-0.885 2.825-1.205 1.43-0.445 2.925-0.685 4.42-0.715z"></path><path fill="#8dc742" style="fill: var(--color23, #8dc742)" d="M15.87 10.86c2.565-0.050 5.135 0.985 6.95 2.805 0.53 0.52 1.020 1.090 1.4 1.735 0.235 0.4 0.255 0.9 0.115 1.34-0.13 0.41-0.475 0.72-0.86 0.905-0.525 0.235-1.17 0.15-1.615-0.21-0.165-0.135-0.305-0.3-0.42-0.475-0.275-0.41-0.6-0.785-0.95-1.13-0.885-0.81-1.98-1.415-3.165-1.645-0.495-0.105-1.005-0.125-1.51-0.125-1.285-0.005-2.54 0.46-3.595 1.175-0.46 0.335-0.885 0.72-1.25 1.155-0.26 0.3-0.46 0.645-0.725 0.945-0.415 0.395-1.055 0.55-1.6 0.36-0.49-0.175-0.915-0.58-1.035-1.095-0.055-0.265-0.105-0.54-0.010-0.805 0.055-0.23 0.18-0.435 0.31-0.63 0.34-0.505 0.73-0.975 1.15-1.415 0.495-0.505 1.065-0.935 1.65-1.33 0.575-0.375 1.195-0.685 1.84-0.93 1.055-0.415 2.19-0.595 3.32-0.63z"></path><path fill="#8dc742" style="fill: var(--color23, #8dc742)" d="M15.76 17.060c1.665-0.11 3.345 0.73 4.3 2.095 0.5 0.73 0.835 1.585 0.885 2.47 0.020 0.425 0.025 0.855-0.035 1.28-0.215 1.075-0.76 2.090-1.58 2.825-0.665 0.59-1.48 1.025-2.36 1.185-0.55 0.095-1.115 0.145-1.67 0.060-0.62-0.080-1.22-0.305-1.765-0.6-0.57-0.32-1.055-0.765-1.47-1.265-0.315-0.415-0.57-0.875-0.76-1.36-0.35-1.010-0.425-2.135-0.090-3.16 0.15-0.475 0.355-0.94 0.65-1.345 0.35-0.51 0.78-0.975 1.3-1.31 0.755-0.54 1.67-0.83 2.595-0.875zM15.8 19.425c-0.28 0.035-0.56 0.095-0.83 0.19-0.295 0.135-0.56 0.33-0.805 0.545-0.185 0.155-0.31 0.37-0.44 0.575-0.41 0.655-0.455 1.5-0.19 2.215 0.175 0.44 0.455 0.84 0.82 1.14 0.31 0.215 0.65 0.4 1.020 0.49 0.295 0.060 0.605 0.055 0.91 0.035 0.435-0.025 0.85-0.19 1.2-0.445 0.595-0.385 0.97-1.045 1.095-1.73 0.030-0.48 0.020-0.98-0.175-1.425-0.125-0.26-0.28-0.505-0.455-0.725-0.55-0.58-1.345-0.945-2.15-0.865z"></path></symbol>',
				'pp-podcastaddict' => '<symbol id="icon-pp-podcastaddict" viewBox="0 0 32 32"><defs><linearGradient id="b"><stop offset="0" stop-color="#E85D27"/><stop offset="1" stop-color="#F0772B"/></linearGradient></defs><path fill="#f4842d" style="fill: var(--color24, #f4842d)" d="M7.211 0h17.579c3.982 0 7.211 3.228 7.211 7.211v17.579c0 3.982-3.228 7.211-7.211 7.211h-17.579c-3.982 0-7.211-3.228-7.211-7.211v-17.579c0-3.982 3.228-7.211 7.211-7.211z"></path><path fill="url(#b)" d="M32 24.733v-6.94l-7.406-7.406-6.148-2.911-3.074-1.467s-3.773 2.515-4.471 2.933-2.865 6.080-3.284 7.057 1.185 7.324 1.185 7.324l8.623 8.676h7.311c4.012-0.001 7.264-3.253 7.265-7.265v0z"></path><path fill="#fff" style="fill: var(--color8, #fff)" d="M16 5.729c-5.673 0.001-10.271 4.599-10.271 10.272 0 3.879 2.15 7.256 5.323 9.004l0.053 0.027v-9.031c0-2.377 1.694-4.358 3.941-4.802l0.031-0.005v1.733c-1.334 0.41-2.287 1.631-2.287 3.074 0 1.773 1.437 3.21 3.21 3.21 1.444 0 2.665-0.953 3.068-2.264l0.006-0.023h1.733c-0.449 2.278-2.43 3.972-4.808 3.972-1.231 0-2.355-0.454-3.215-1.204l0.006 0.005v6.063c0.957 0.325 2.059 0.512 3.205 0.512 2.159 0 4.162-0.665 5.816-1.801l-0.035 0.023v1.779h4.494v-10.271c0-5.672-4.598-10.271-10.271-10.271v0zM16 17.885c-1.041 0-1.885-0.844-1.885-1.885s0.844-1.885 1.885-1.885c1.041 0 1.885 0.844 1.885 1.885v0c0 0 0 0 0 0.001 0 1.041-0.844 1.885-1.885 1.885 0 0 0 0-0.001 0v0zM19.21 16c0-1.773-1.437-3.21-3.21-3.21v-1.685c2.703 0 4.894 2.191 4.894 4.894v0zM22.077 16c0-3.356-2.721-6.077-6.077-6.077v0-1.666c4.276 0 7.743 3.467 7.743 7.743v0z"></path></symbol>',
				'pp-podchaser'     => '<symbol id="icon-pp-podchaser" viewBox="0 0 32 32"><path fill="#f70664" style="fill: var(--color25, #f70664)" d="M16 32c-8.85 0-16-7.175-16-16.055 0-8.875 7.15-16.045 16-16.045s16 7.17 16 16.045c0 8.88-7.15 16.055-16 16.055z"></path><path fill="#fff" style="fill: var(--color8, #fff)" d="M16 29.755c-7.61 0-13.76-6.175-13.76-13.81 0-7.63 6.15-13.8 13.76-13.8s13.76 6.17 13.76 13.8c0 7.635-6.15 13.81-13.76 13.81z"></path><path fill="#293b65" style="fill: var(--color26, #293b65)" d="M16 26.545c-5.66 0-10.24-4.595-10.24-10.275s4.58-10.275 10.24-10.275c5.665 0 10.24 4.595 10.24 10.275s-4.575 10.275-10.24 10.275z"></path><path fill="#293b65" style="fill: var(--color26, #293b65)" d="M11.52 29.005c-2.235-0.77-4.21-2.105-5.76-3.835v-8.9h5.76v12.735z"></path></symbol>',
				'pp-radiopublic'   => '<symbol id="icon-pp-radiopublic" viewBox="0 0 32 32"><path fill="#cf2327" style="fill: var(--color30, #cf2327)" d="M3.365 28.635c-2.455-2.455-3.365-6.455-3.365-12.635s0.91-10.18 3.365-12.635c2.455-2.455 6.455-3.365 12.635-3.365s10.18 0.91 12.635 3.365c2.455 2.455 3.365 6.455 3.365 12.635s-0.91 10.18-3.365 12.635c-2.455 2.455-6.455 3.365-12.635 3.365s-10.18-0.91-12.635-3.365z"></path><path fill="#fff" style="fill: var(--color8, #fff)" d="M14.995 5.435c0.22-0.010 0.44-0.010 0.66 0.005 0.225 0.015 0.445 0.040 0.66 0.085 0.22 0.040 0.435 0.095 0.645 0.16 0.21 0.070 0.415 0.15 0.62 0.245 0.010 1.030-0.005 2.060 0.005 3.095 1.785-0.195 3.485-0.865 5.285-0.975 0.23-0.020 0.465-0.025 0.7-0.010 0.235 0.010 0.47 0.035 0.7 0.080s0.46 0.1 0.68 0.175c0.225 0.075 0.44 0.16 0.655 0.265-0.015 5.84-0.015 11.675-0.005 17.515-0.26-0.12-0.53-0.22-0.805-0.295-0.28-0.080-0.56-0.14-0.845-0.175-0.285-0.040-0.57-0.060-0.855-0.055-0.29 0.005-0.575 0.030-0.86 0.075-1.65 0.22-3.235 0.815-4.9 0.925-0.24 0.020-0.475 0.030-0.715 0.015-0.24-0.010-0.475-0.035-0.71-0.075s-0.465-0.095-0.695-0.17c-0.225-0.070-0.45-0.155-0.665-0.255-0.015-1.030 0-2.065-0.005-3.095-1.51 0.15-2.95 0.655-4.445 0.875-0.245 0.045-0.49 0.080-0.735 0.1s-0.495 0.025-0.74 0.015c-0.245-0.010-0.495-0.035-0.735-0.070-0.245-0.035-0.49-0.090-0.725-0.155-0.26-0.065-0.5-0.18-0.75-0.27-0.010-0.345-0.015-0.695-0.010-1.045 0.005-5.49-0.010-10.975 0.005-16.465 0.16 0.050 0.315 0.105 0.465 0.17 0.21 0.070 0.42 0.125 0.635 0.175 0.215 0.045 0.43 0.080 0.645 0.105 0.22 0.025 0.435 0.040 0.655 0.040s0.44-0.010 0.655-0.030c1.865-0.18 3.645-0.91 5.53-0.98zM12.095 8.94c-0.21 0.055-0.425 0.105-0.64 0.155s-0.43 0.095-0.65 0.135c-0.215 0.040-0.435 0.075-0.65 0.105-0.22 0.035-0.44 0.060-0.66 0.085-0.015 0.335-0.020 0.67-0.015 1.005 0.010 3.475-0.010 6.95 0.010 10.425 0.17-0.015 0.34-0.035 0.51-0.055s0.34-0.045 0.51-0.075c0.17-0.030 0.335-0.065 0.505-0.1 0.165-0.035 0.335-0.075 0.5-0.12-0.010-1.625-0.010-3.25-0.005-4.875 0.065-0.020 0.125-0.040 0.19-0.055 0.060-0.015 0.125-0.030 0.19-0.040 0.065-0.015 0.125-0.020 0.19-0.030 0.065-0.005 0.13-0.010 0.195-0.010 0.19 0.075 0.205 0.305 0.2 0.485 0.020 1.19-0.015 2.385 0.010 3.58 0 0.060 0 0.12 0.005 0.18 0.010 0.060 0.020 0.12 0.035 0.18 0.010 0.060 0.030 0.12 0.050 0.175 0.020 0.060 0.045 0.115 0.070 0.17 0.17-0.050 0.34-0.090 0.51-0.13s0.345-0.070 0.515-0.1c0.175-0.030 0.345-0.055 0.52-0.075s0.35-0.035 0.525-0.045c-0.025-0.055-0.050-0.115-0.070-0.18-0.015-0.060-0.035-0.12-0.045-0.18-0.015-0.065-0.025-0.125-0.035-0.19s-0.015-0.125-0.015-0.19c-0.015-1.27 0.005-2.54-0.005-3.81 0.005-0.080 0.005-0.16 0-0.24s-0.015-0.16-0.030-0.24c-0.015-0.075-0.030-0.155-0.055-0.23-0.020-0.080-0.050-0.155-0.080-0.23-0.040-0.065-0.080-0.125-0.13-0.18-0.045-0.060-0.1-0.11-0.155-0.155-0.060-0.050-0.12-0.090-0.185-0.13-0.065-0.035-0.135-0.065-0.205-0.090 0.055-0.035 0.105-0.075 0.155-0.12 0.050-0.040 0.095-0.085 0.14-0.13 0.050-0.045 0.090-0.095 0.135-0.145 0.040-0.050 0.080-0.1 0.115-0.15 0.070-0.12 0.13-0.245 0.18-0.37s0.085-0.26 0.115-0.39c0.030-0.135 0.050-0.27 0.055-0.405 0.010-0.14 0.005-0.275-0.010-0.41 0.010-0.185 0.015-0.365 0.015-0.55s-0.005-0.365-0.020-0.55c-0.010-0.18-0.030-0.365-0.055-0.545-0.025-0.185-0.055-0.365-0.090-0.545-0.030-0.085-0.065-0.165-0.11-0.245-0.045-0.075-0.095-0.145-0.16-0.21-0.060-0.065-0.13-0.125-0.205-0.175-0.075-0.045-0.155-0.085-0.235-0.115-0.14-0.030-0.275-0.045-0.415-0.055-0.135-0.005-0.275-0.005-0.41 0.005-0.14 0.010-0.275 0.030-0.41 0.060s-0.27 0.070-0.4 0.12v0zM11.505 10.855c0.070-0.030 0.14-0.050 0.215-0.070 0.070-0.015 0.145-0.030 0.22-0.035 0.075-0.010 0.15-0.015 0.225-0.015 0.075 0.005 0.15 0.010 0.225 0.020 0.020 0.050 0.035 0.1 0.050 0.15s0.025 0.1 0.030 0.15c0.010 0.055 0.015 0.105 0.015 0.16 0 0.050 0 0.105-0.005 0.155 0 0.56 0.010 1.12-0.005 1.68-0.005 0.050-0.015 0.1-0.030 0.15s-0.035 0.1-0.065 0.145c-0.025 0.045-0.060 0.085-0.1 0.12s-0.080 0.065-0.13 0.090c-0.050 0.015-0.1 0.035-0.15 0.050-0.055 0.015-0.105 0.030-0.16 0.040-0.050 0.015-0.105 0.025-0.155 0.035-0.055 0.010-0.11 0.020-0.16 0.025-0.035-0.95-0.005-1.9-0.020-2.85zM20.545 11.455c-0.245 0.065-0.49 0.125-0.735 0.18s-0.49 0.11-0.735 0.16c-0.25 0.050-0.495 0.095-0.745 0.135-0.245 0.045-0.495 0.080-0.745 0.115-0.010 3.8 0 7.595-0.005 11.39v0.040c0.675-0.075 1.345-0.22 2.015-0.345 0.035-1.54-0.010-3.075 0.020-4.615 0.125-0.020 0.25-0.045 0.375-0.070s0.25-0.055 0.375-0.085c0.12-0.030 0.245-0.065 0.365-0.105 0.125-0.035 0.245-0.075 0.365-0.12 0.1-0.035 0.195-0.080 0.285-0.13 0.095-0.050 0.18-0.105 0.265-0.17 0.085-0.060 0.165-0.13 0.24-0.2 0.075-0.075 0.145-0.155 0.21-0.235 0.065-0.095 0.125-0.195 0.175-0.3s0.095-0.21 0.135-0.32c0.035-0.11 0.070-0.22 0.090-0.335 0.025-0.11 0.040-0.225 0.045-0.345 0.045-0.77 0.005-1.545 0.020-2.32 0.010-0.13 0.015-0.26 0.010-0.39 0-0.135-0.005-0.265-0.015-0.395s-0.030-0.26-0.050-0.39c-0.020-0.13-0.045-0.26-0.075-0.39-0.020-0.070-0.045-0.135-0.080-0.2-0.030-0.065-0.070-0.13-0.115-0.185-0.040-0.060-0.090-0.115-0.145-0.165-0.050-0.050-0.11-0.095-0.17-0.135-0.11-0.045-0.22-0.080-0.335-0.105s-0.23-0.045-0.345-0.050c-0.12-0.005-0.235 0-0.35 0.010-0.12 0.015-0.235 0.035-0.345 0.070v0zM19.61 13.45c0.055-0.020 0.11-0.035 0.165-0.050s0.11-0.025 0.17-0.035c0.055-0.010 0.11-0.015 0.17-0.020 0.055-0.005 0.115-0.005 0.17-0.005 0.185 0.040 0.23 0.265 0.205 0.42 0.010 0.705 0 1.405 0.005 2.105 0.010 0.050 0.010 0.095 0.010 0.145-0.005 0.045-0.015 0.095-0.025 0.14-0.015 0.045-0.035 0.090-0.055 0.13-0.025 0.040-0.050 0.080-0.085 0.115-0.175 0.215-0.48 0.19-0.72 0.245-0.025-1.065-0.010-2.125-0.010-3.19z"></path><path fill="#d2d3d5" style="fill: var(--color31, #d2d3d5)" d="M14.545 22.97c0.16-0.005 0.32-0.010 0.475-0.005 0.16 0.005 0.32 0.010 0.48 0.025 0.16 0.010 0.315 0.030 0.475 0.050 0.155 0.025 0.315 0.050 0.47 0.085 0.095 0.020 0.195 0.045 0.29 0.070s0.19 0.050 0.285 0.080c0.095 0.030 0.19 0.060 0.28 0.095 0.095 0.035 0.185 0.070 0.28 0.105l-0.005 0.040c-0.005 0.030-0.015 0.060-0.020 0.090-0.010 0.030-0.020 0.060-0.035 0.085-0.010 0.030-0.025 0.060-0.040 0.085s-0.030 0.050-0.050 0.080c-0.090 0.12-0.185 0.235-0.285 0.345-0.1 0.115-0.205 0.22-0.315 0.325s-0.225 0.2-0.345 0.295c-0.115 0.095-0.24 0.185-0.365 0.265-0.125 0.090-0.25 0.18-0.38 0.265-0.125 0.090-0.255 0.17-0.39 0.255-0.13 0.080-0.26 0.16-0.395 0.235-0.135 0.080-0.27 0.155-0.405 0.225-0.015-1.020-0.005-2.075-0.005-3.095z"></path></symbol>',
				'pp-rss'           => '<symbol id="icon-pp-rss" viewBox="0 0 32 32"><path fill="#ffa500" style="fill: var(--color40, #ffa500)" d="M6 0h20c3.317 0 6 2.683 6 6v20c0 3.317-2.683 6-6 6h-20c-3.317 0-6-2.683-6-6v-20c0-3.317 2.683-6 6-6z"></path><path fill="#fff" style="fill: var(--color8, #fff)" d="M8 28c-2.213 0-4-1.788-4-4s1.787-4 4-4c2.213 0 4 1.788 4 4s-1.787 4-4 4z"></path><path fill="#fff" style="fill: var(--color8, #fff)" d="M4 16c1.575 0 3.138 0.31 4.593 0.913 1.455 0.605 2.777 1.488 3.893 2.602s1.998 2.438 2.603 3.892c0.602 1.455 0.913 3.017 0.913 4.593h4c0-2.1-0.415-4.183-1.218-6.122-0.805-1.942-1.982-3.705-3.47-5.19-1.485-1.488-3.248-2.665-5.19-3.47-1.94-0.803-4.022-1.217-6.122-1.217v4z"></path><path fill="#fff" style="fill: var(--color8, #fff)" d="M4 8c2.627 0 5.227 0.518 7.652 1.523 2.428 1.005 4.633 2.477 6.49 4.335s3.33 4.063 4.335 6.49c1.005 2.425 1.523 5.025 1.523 7.652h4c0-3.152-0.62-6.273-1.828-9.185-1.205-2.91-2.973-5.558-5.203-7.785-2.227-2.23-4.875-3.998-7.785-5.202-2.912-1.208-6.032-1.827-9.185-1.827v4z"></path></symbol>',
				'pp-soundcloud'    => '<symbol id="icon-pp-soundcloud" viewBox="0 0 74 32"><defs><linearGradient id="b"><stop offset="0" stop-color="#f79810"/><stop offset="1" stop-color="#f8310e"/></linearGradient></defs><path fill="url(#b)" d="M0.834 19.139c-0.154 0-0.276 0.119-0.296 0.288l-0.539 4.093 0.539 4.022c0.020 0.17 0.142 0.288 0.296 0.288 0.149 0 0.27-0.117 0.294-0.286v0l0.639-4.024-0.639-4.094c-0.023-0.168-0.146-0.287-0.294-0.287zM3.864 16.902c-0.025-0.176-0.151-0.297-0.307-0.297s-0.285 0.125-0.307 0.297c0 0.002-0.724 6.617-0.724 6.617l0.724 6.471c0.021 0.174 0.15 0.299 0.307 0.299 0.155 0 0.28-0.122 0.306-0.297l0.824-6.472-0.823-6.617zM15.278 10.798c-0.297 0-0.543 0.243-0.557 0.552l-0.577 12.175 0.577 7.868c0.015 0.308 0.26 0.55 0.557 0.55 0.296 0 0.54-0.243 0.558-0.552v0.002l0.652-7.869-0.652-12.175c-0.018-0.309-0.262-0.552-0.558-0.552zM9.371 15.030c-0.228 0-0.414 0.182-0.433 0.425l-0.65 8.067 0.65 7.805c0.017 0.242 0.204 0.424 0.433 0.424 0.226 0 0.412-0.182 0.432-0.424l0.739-7.805-0.739-8.068c-0.020-0.242-0.206-0.424-0.432-0.424zM21.282 31.942c0.368 0 0.668-0.297 0.683-0.677l0.566-7.736-0.565-16.208c-0.015-0.38-0.314-0.678-0.683-0.678-0.372 0-0.672 0.298-0.684 0.678 0 0.001-0.502 16.207-0.502 16.207l0.502 7.739c0.012 0.377 0.312 0.675 0.683 0.675zM33.572 31.979c0.507 0 0.926-0.418 0.935-0.931v0l0.395-7.514-0.395-18.72c-0.009-0.513-0.428-0.931-0.935-0.931s-0.927 0.418-0.935 0.932l-0.353 18.713c0 0.012 0.353 7.525 0.353 7.525 0.008 0.509 0.428 0.926 0.935 0.926zM27.38 31.949c0.441 0 0.797-0.354 0.809-0.806v0.006l0.48-7.618-0.481-16.326c-0.012-0.452-0.368-0.805-0.809-0.805-0.445 0-0.8 0.353-0.81 0.805l-0.427 16.327 0.428 7.617c0.009 0.447 0.365 0.801 0.81 0.801zM12.313 31.878c0.262 0 0.475-0.21 0.494-0.487l0.696-7.869-0.696-7.486c-0.019-0.276-0.232-0.485-0.494-0.485-0.266 0-0.479 0.21-0.495 0.488l-0.613 7.483 0.613 7.869c0.018 0.277 0.229 0.487 0.495 0.487zM6.452 31.447c0.192 0 0.347-0.151 0.369-0.361l0.781-7.566-0.78-7.851c-0.023-0.21-0.178-0.361-0.37-0.361-0.194 0-0.349 0.152-0.369 0.361 0 0.001-0.688 7.851-0.688 7.851l0.688 7.564c0.019 0.21 0.174 0.362 0.369 0.362zM30.464 6.933c-0.479 0-0.864 0.381-0.873 0.869l-0.39 15.73 0.39 7.569c0.009 0.482 0.393 0.863 0.873 0.863 0.479 0 0.861-0.38 0.872-0.869v0.006l0.438-7.569-0.438-15.732c-0.011-0.486-0.393-0.868-0.872-0.868zM18.269 31.958c0.331 0 0.605-0.27 0.62-0.615l0.609-7.816-0.609-14.959c-0.017-0.345-0.289-0.615-0.62-0.615-0.334 0-0.607 0.27-0.621 0.615 0 0.001-0.539 14.959-0.539 14.959l0.54 7.819c0.013 0.342 0.286 0.612 0.62 0.612zM25.064 31.221v-0.004l0.523-7.686-0.523-16.755c-0.013-0.416-0.341-0.742-0.746-0.742-0.407 0-0.735 0.326-0.746 0.742l-0.464 16.754 0.465 7.69c0.011 0.412 0.339 0.738 0.746 0.738s0.732-0.326 0.746-0.74l-0 0.003zM64.62 13.872c-1.242 0-2.427 0.251-3.506 0.704-0.721-8.168-7.571-14.576-15.926-14.576-2.044 0-4.037 0.403-5.797 1.083-0.684 0.265-0.866 0.537-0.872 1.066v28.765c0.007 0.555 0.438 1.017 0.979 1.071 0.023 0.002 24.96 0.015 25.122 0.015 5.005 0 9.063-4.058 9.063-9.064s-4.057-9.064-9.062-9.064zM36.666 2.124c-0.544 0-0.991 0.447-0.999 0.996l-0.409 20.416 0.41 7.41c0.007 0.541 0.454 0.987 0.998 0.987 0.543 0 0.99-0.446 0.998-0.995v0.009l0.445-7.411-0.445-20.418c-0.008-0.548-0.455-0.995-0.998-0.995z"></path></symbol>',
				'pp-www'           => '<symbol id="icon-pp-www" viewBox="0 0 32 32"><path d="M16 0c-8.837 0-16 7.163-16 16s7.163 16 16 16 16-7.163 16-16-7.163-16-16-16zM2.051 17h3.431c0.076 1.66 0.327 3.179 0.717 4.543h-3.049c-0.611-1.41-0.985-2.941-1.099-4.543zM20.936 15c-0.075-1.607-0.266-3.13-0.524-4.543h3.278c0.436 1.342 0.736 2.855 0.826 4.543h-3.58zM13.478 21.544c-0.282-1.558-0.421-3.080-0.446-4.543h5.937c-0.026 1.464-0.164 2.985-0.447 4.543h-5.044zM18.086 23.544c-0.469 1.8-1.148 3.635-2.087 5.474-0.939-1.839-1.617-3.674-2.086-5.474h4.173zM13.059 15c0.079-1.617 0.284-3.14 0.555-4.543h4.773c0.271 1.404 0.476 2.927 0.555 4.543h-5.883zM14.063 8.457c0.625-2.44 1.397-4.365 1.937-5.55 0.54 1.185 1.312 3.11 1.937 5.55h-3.874zM19.988 8.457c-0.638-2.644-1.457-4.768-2.065-6.141 0.756 0.538 3.267 2.526 4.98 6.141h-2.914zM14.077 2.316c-0.608 1.373-1.427 3.497-2.065 6.141h-2.914c1.713-3.615 4.224-5.603 4.98-6.141zM11.588 10.457c-0.258 1.414-0.448 2.936-0.524 4.543h-3.581c0.090-1.689 0.39-3.202 0.826-4.543h3.278zM5.484 15h-3.433c0.114-1.603 0.489-3.134 1.099-4.543h3.063c-0.393 1.364-0.648 2.882-0.729 4.543zM7.483 17h3.554c0.023 1.466 0.146 2.986 0.408 4.543h-3.134c-0.44-1.345-0.74-2.858-0.828-4.543zM11.859 23.544c0.491 2.036 1.219 4.118 2.269 6.208-0.708-0.507-3.283-2.533-5.029-6.208h2.76zM17.87 29.756c1.051-2.091 1.78-4.174 2.271-6.212h2.78c-1.741 3.694-4.327 5.702-5.052 6.212zM20.556 21.544c0.262-1.558 0.385-3.077 0.408-4.543h3.555c-0.086 1.688-0.382 3.201-0.815 4.543h-3.147zM26.518 17h3.432c-0.114 1.603-0.489 3.134-1.099 4.543h-3.049c0.39-1.364 0.641-2.883 0.717-4.543zM26.516 15c-0.080-1.662-0.336-3.18-0.729-4.543h3.063c0.611 1.41 0.985 2.94 1.099 4.543h-3.434zM27.775 8.457h-2.693c-0.894-2.153-2.059-3.857-3.156-5.122 2.399 1.127 4.426 2.909 5.85 5.122zM10.074 3.335c-1.097 1.265-2.262 2.969-3.156 5.122h-2.693c1.423-2.213 3.45-3.995 5.849-5.122zM4.225 23.544h2.677c0.88 2.131 2.029 3.829 3.119 5.096-2.375-1.129-4.383-2.899-5.795-5.096zM21.98 28.639c1.089-1.267 2.238-2.965 3.118-5.096h2.677c-1.412 2.197-3.42 3.967-5.795 5.096z"></path></symbol>',
				'pp-deezer'       => '<symbol id="icon-pp-deezer" viewBox="0 0 46 32">
				<defs>
				<linearGradient id="a"><stop offset="0" stop-color="#358c7b"/><stop offset=".526" stop-color="#33a65e"/></linearGradient><linearGradient id="c"><stop offset="0" stop-color="#222b90"/><stop offset="1" stop-color="#367b99"/></linearGradient><linearGradient id="d"><stop offset="0" stop-color="#f90"/><stop offset="1" stop-color="#ff8000"/></linearGradient><linearGradient id="e"><stop offset="0" stop-color="#ff8000"/><stop offset="1" stop-color="#cc1953"/></linearGradient><linearGradient id="f"><stop offset="0" stop-color="#cc1953"/><stop offset="1" stop-color="#241284"/></linearGradient><linearGradient id="g"><stop offset="0" stop-color="#222b90"/><stop offset="1" stop-color="#3559a6"/></linearGradient><linearGradient id="h"><stop offset="0" stop-color="#cc1953"/><stop offset="1" stop-color="#241284"/></linearGradient><linearGradient id="i"><stop offset=".003" stop-color="#fc0"/><stop offset="1" stop-color="#ce1938"/></linearGradient><linearGradient id="j"><stop offset=".003" stop-color="#ffd100"/><stop offset="1" stop-color="#fd5a22"/></linearGradient>
				</defs>
				<path fill="#40ab5d" d="M33.685 3.743h8.333v4.654h-8.333z"></path>
				<path fill="url(#a)" d="M33.685 10.195h8.333v4.654h-8.333z"></path>
				<path fill="url(#c)" d="M33.685 16.648h8.333v4.654h-8.333z"></path>
				<path fill="url(#d)" d="M3.479 23.101h8.333v4.654h-8.333z"></path>
				<path fill="url(#e)" d="M13.541 23.101h8.333v4.654h-8.333z"></path>
				<path fill="url(#f)" d="M23.623 23.101h8.333v4.654h-8.333z"></path>
				<path fill="url(#g)" d="M33.685 23.101h8.333v4.654h-8.333z"></path>
				<path fill="url(#h)" d="M23.623 16.648h8.333v4.654h-8.333z"></path>
				<path fill="url(#i)" d="M13.541 16.648h8.333v4.654h-8.333z"></path>
				<path fill="url(#j)" d="M13.541 10.195h8.333v4.654h-8.333z"></path>
				</symbol>',
			)
		);
	}

	/**
	 * Register hooked functions.
	 *
	 * @since 1.0.0
	 */
	public function register() {
		add_filter( 'wp_footer', array( $this, 'add_icons' ), 9999 );
		add_filter( 'admin_footer', array( $this, 'add_icons' ), 9999 );
	}

	/**
	 * Returns the instance of this class.
	 *
	 * @since  6.0.0
	 *
	 * @return object Instance of this class.
	 */
	public static function get_instance() {
		if ( null === self::$instance ) {
			self::$instance = new self();
			self::$instance->register();
		}
		return self::$instance;
	}
}
